; Das RBM Kaokoveld ist ein physikalisch basiertes, rechner-
; gesttztes Modell zu Berechnung von Biomasse aus
; mesoskaligen Satellitendaten. Diese mssen vor der
; Einspeisung in das Modell in definierte geophysikalische
; Parameter umgerechnet werden.
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Dieses Modell wurde im Rahmen der Promotion von
; J. Richters am Geographischen Institut der Universitt
; Bonn (GIUB) entwickelt.
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PRO RBM_YEAR, home, model_time
; J. Richters, 07/2003, RSRG, GIUB, University of Bonn
; Diese Routine steuert die Berechung aller Prozesse,
; die im Laufe eines Jahres stattfinden
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechung von jhrlichen Zwischenprodukten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Print, model_time
jahr = STRTRIM(model_time(0),2)
steps = model_time(1)
TS_start = model_time(2)
TS_end = model_time(3)
; ---------------------------------------------------------
; Epsilon_max
; ---------------------------------------------------------
EPSILON_MAX, home, jahr
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechung von kurzfristigen Daten (jeweils 10 Tage)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
FOR i=0, steps-1 DO BEGIN
	; -----------------------------------------------------
	; Allgemeine Parameter pro Timestep
	; -----------------------------------------------------
	date=INTARR(2)
	date(0)=model_time(0)
	date(1)=model_time(2)+i
	; -----------------------------------------------------
	; Meldung an Log-Window
	; -----------------------------------------------------
	log = 'Starting Timestep '+ STRTRIM(i+1,2)
	print,log
	; -----------------------------------------------------
	; Biomasse
	; -----------------------------------------------------
	BIOMASS_SINGLE, home, date
	; -----------------------------------------------------
	; Bilanzberecnung
	; -----------------------------------------------------
	bil_param='bio'
	BILANZ, home, date, bil_param
	; -----------------------------------------------------
	; Weidetierdynamik
	; -----------------------------------------------------
	; Biomasse Destruenten (Wild, mirkobiotischer
	; Abbau, etc.)
	; -----------------------------------------------------
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ende der Routine RBM_YEAR.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; *********************************************************
; EPSILON_MAX.pro
; *********************************************************
; Die Routine EPSILON_MAX.pro berechnet den Parameter
; epsilon_max unter Einbeziehung folgender Datenstze:
; - landcover type
; - relief information
; - soil information
; Die Routine nutzt einerseits die Ergebnisse der vorher
; abgearbeiteten Routine LANDCOVER.pro und integriert diese
; ber eine Look-up Tabelle der Biometypen mit den
; beeinflussenden Parametern Relief und Boden.
; Dabei nutzt die Routine EPSILON_MAX.pro zwei Submodule:
; - E_MAX_DGM zur Berechung der Beeinflussung durch
;   das Relief
; - E_MAX_SOIL zur Berechung der Beeinflussung durch
;   Bodendifferenzierung
; *********************************************************
PRO EPSILON_MAX, home, year
; home=string(Basisverzeichnis des Modelllaufs)
; year=intarr(1), gibt jahr der prozessierung an
jahr=year
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Konstruktion der Datenpfade
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
LUCC_pfad=	home+'\'+jahr+'\ysp\YDP_VCF_'+jahr+'.data'
DGM_pfad=	home+'\'+jahr+'\ysp\CDP_DGM_'+jahr+'.data'
SOIL_pfad=	home+'\'+jahr+'\ysp\CDP_SOIL_'+jahr+'.data'
BCF_PFD =	home+'\'+jahr+'\ysp\YSP_BCF_'+jahr+'.data'
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der LUCC Daten und Auslesender Dateiinformation
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, LUCC_pfad, r_fid=LUCC_id, /no_realize
  if (LUCC_id eq -1) then return
ENVI_FILE_QUERY, LUCC_id, ns=ns, nl=nl, nb=nb
map_info=ENVI_GET_MAP_INFO(fid=LUCC_id)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der Reliefeinflsse
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DGM_fuzzy=FLTARR(ns,nl)
DGM_fuzzy(*,*)=1
SLP_fuzzy=FLTARR(ns,nl)
SLP_fuzzy(*,*)=1
dgm=FILE_TEST(DGM_pfad)
if (dgm eq 1) then begin
	E_MAX_DGM, DGM_pfad, DGM_fuzzy, SLP_fuzzy
endif
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der Bodeneinflsse
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
SOIL_fuzzy=FLTARR(ns,nl)
SOIL_fuzzy(*,*)=1
soil=FILE_TEST(SOIL_pfad)
if (soil eq 1) then begin
	E_MAX_SOIL, SOIL_pfad, SOIL_fuzzy
endif
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Look-Up Tabelle fr Biome Epsilon
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BARREN= 0.003 		;
GRASS=  0.00804		;
TREE=	0.01116		;
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ffnen und Einlesen der LUCC Daten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
VCF=BYTARR(ns,nl,3)
dims = [-1, 0, ns-1, 0, nl-1]
FOR i=0,2 DO BEGIN
VCF(*,*,i) = ENVI_GET_DATA(fid=LUCC_id, dims=dims, pos=i)
ENDFOR
ENVI_FILE_MNG, id=LUCC_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Zuordnung der Look-Up Tabelle
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BIOME_fuzzy=FLTARR(ns,nl)
; ---------------------------------------------------------
FOR i=0,2 DO BEGIN
  dum = VCF(*,*,i)
  hlp = WHERE(dum GT 100, count)
   IF count GT 0 THEN dum(hlp)=0
  VCF(*,*,i) = dum
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechung von Epsilon aus den Flchenbedeckungen
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
VCF = FLOAT(VCF)
VCF(*,*,0) =  VCF(*,*,0) * BARREN
VCF(*,*,1) =  VCF(*,*,1) * GRASS
VCF(*,*,2) =  VCF(*,*,2) * TREE
BIOME_fuzzy = VCF(*,*,0)+VCF(*,*,1)+VCF(*,*,2)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Zusammenfhrung der BIOME mit DGM_fuzzy und SOIL_fuzzy
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Gewichtungen der Einfluparameter Relief & Boden
x_relief =	0.8
;w_Slope =	0.5
w_Soil =	0.6
Relief_fuzzy=FLTARR(ns,nl)
FUZZY_AND, DGM_fuzzy, SLP_fuzzy, Relief_fuzzy
Epsi_all=FLTARR(ns,nl,2)
Epsi_all(*,*,0)=Relief_fuzzy(*,*) * x_relief
Epsi_all(*,*,1)=SOIL_fuzzy(*,*) * w_Soil
Epsilon_ges=FLTARR(ns,nl)
Epsilon_ges(*,*)=MEAN(Epsi_all(*,*,*))
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Integration der Biometypes
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Epsilon_gesamt=FLTARR(ns,nl)
Epsilon_gesamt(*,*)=Epsilon_ges(*,*) * BIOME_fuzzy(*,*)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Integration der Seamask
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, DGM_pfad, r_fid=DGM_id, /no_realize
  if (DGM_id eq -1) then return
ENVI_FILE_QUERY, DGM_id, ns=ns, nl=nl, nb=nb
map_info=ENVI_GET_MAP_INFO(fid=DGM_id)
seamask=INTARR(ns,nl)
dims = [-1, 0, ns-1, 0, nl-1]
seamask(*,*) = ENVI_GET_DATA(fid=DGM_id, dims=dims, pos=0)
ENVI_FILE_MNG, id=DGM_id,/remove
meer=WHERE(seamask eq 0,meer_count)
IF meer_count gt 0 THEN Epsilon_gesamt(meer)=-1
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe des Parameters EPSILON_MAX
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bnames = ['Biophysical Conversion Factor [MJ/g C]']
openw,1, BCF_PFD
writeu,1, Epsilon_gesamt
close,1
ENVI_SETUP_HEAD, fname=BCF_PFD, $
  ns=ns, nl=nl, nb=1, map_info=map_info,$
  interleave=0, data_type=4, bnames=bnames,$
  offset=0, /write, r_fid=out_id
ENVI_FILE_MNG, id=out_id,/remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ende der Routine EPSILON_MAX.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; *********************************************************
; E_MAX_DGM.pro
; Diese Routine berechnet fr die bergeordnete Routine
; EPSILON_MAX.pro den Einfluss des Reliefs fr die
; Biome-Epsilonwerte wird aufgerufen von EPSILON_MAX
; Benutze Variablen:
; - DGM_Pfad: STRING-Variable mit Pfadangabe zum DGM
; - DGM_fuzzy: 2-D FLOAT-ARRAY zur Aufnahme der Werte,
;   per default auf 1 gesetzt
; - SLP_fuzzy: 2-D FLOAT-ARRAY zur Aufnahme der Werte,
;	per default auf 1 gesetzt
; *********************************************************
PRO E_MAX_DGM, DGM_pfad, DGM_fuzzy, SLP_fuzzy
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der DGM-Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, DGM_pfad, r_fid=dgm_id, /no_realize
if (dgm_id eq -1) then return
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung grundlegender Dateiinformationen
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_FILE_QUERY, dgm_id, ns=ns, nl=nl, nb=nb
map_info = ENVI_GET_MAP_INFO(fid=dgm_id)
dims = [-1l, 0, ns-1, 0, nl-1]
proj = ENVI_GET_PROJECTION(fid=dgm_id, $
	pixel_size=pixel_size)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ableitung der Hangneigung (Slope)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  pos  = [0]
  bptr = [0]
ENVI_DOIT, 'topo_doit', $
    fid=dgm_id, pos=pos, dims=dims, $
    bptr=bptr, /in_memory, $
    pixel_size=pixel_size, r_fid=slope_id
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der DGM- und Slope-Daten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
dgm=FLTARR(ns,nl,2)
dgm(*,*,0) = ENVI_GET_DATA(fid=dgm_id, dims=dims, pos=0)
dgm(*,*,1) = ENVI_GET_DATA(fid=slope_id, dims=dims, pos=0)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Schlieen der offene Dateien
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_FILE_MNG, id=dgm_id, /remove
ENVI_FILE_MNG, id=slope_id, /remove, /delete
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Fuzzyfizierung und Fuzzy-Parameter
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
a1=		-0.0004
b1=		1.4
gu1=	1000
go1=	2200
a2=		-0.2
b2=		1.6
gu2=	3
go2=	8
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; DGM_Fuzzyfizierung & Slope-Fuzzyifizierung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DGM_fuzzy=FLTARR(ns,nl)
DGM_fuzzy(*,*)=a1*dgm(*,*,0)+b1
	t1=WHERE(dgm(*,*,0) lt gu1,count_t1)
	if (count_t1 gt 0) then DGM_fuzzy(t1)=1
	t2=WHERE(dgm(*,*,0) gt go1,count_t2)
	if (count_t2 gt 0) then DGM_fuzzy(t2)=0.3
; ---------------------------------------------------------
SLP_fuzzy=FLTARR(ns,nl)
SLP_fuzzy(*,*)=a2*dgm(*,*,1)+b2
	t3=WHERE(dgm(*,*,1) lt gu2,count_t3)
	if (count_t3 gt 0) then SLP_fuzzy(t3)=1
	t4=WHERE(dgm(*,*,1) gt go2,count_t4)
	if (count_t4 gt 0) then SLP_fuzzy(t4)=0.6
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ende der Routine E_MAX_DGM.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; *********************************************************
; E_MAX_SOIL.pro
; Diese Routine berechnet fr die bergeordnete Routine
; EPSILON_MAX.pro den Einfluss des Bodens fr die
; Biome-Epsilonwerte. Dabei wird die FAO-Weltbodenkarte
; zugunde gelegt.
; Wird aufgerufen von EPSILON_MAX
; Benutze Variablen:
; - SOIL_Pfad: STRING-Variable mit Pfadangabe zu den
;   Soildaten
; - DGM_fuzzy: 2-D FLOAT-ARRAY zur Aufnahme der Werte,
;   per default auf 1 gesetzt
; *********************************************************
PRO E_MAX_SOIL, SOIL_pfad, SOIL_fuzzy
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der DGM-Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, SOIL_pfad, r_fid=soil_id, /no_realize
IF (soil_id EQ -1) THEN RETURN
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung grundlegender Dateiinformationen
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_FILE_QUERY, soil_id, ns=ns, nl=nl, nb=nb
map_info = ENVI_GET_MAP_INFO(fid=soil_id)
dims = [-1l, 0, ns-1, 0, nl-1]
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Auslesen der Bodendaten aus SOIL_pfad
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
soildat=FLTARR(ns,nl,4)
soildat(*,*,2)=ENVI_GET_DATA(fid=soil_id, dims=dims, pos=0)
soildat(*,*,1)=ENVI_GET_DATA(fid=soil_id, dims=dims, pos=1)
soildat(*,*,0)=ENVI_GET_DATA(fid=soil_id, dims=dims, pos=2)
soildat(*,*,3)=ENVI_GET_DATA(fid=soil_id, dims=dims, pos=14)
ENVI_FILE_MNG, id=soil_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Texturbestimmung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
textur=BYTARR(ns,nl)
tex_fuzzy=FLTARR(ns,nl)
;----------------------------------------------------------
coarse=WHERE((soildat(*,*,0) gt 0) and $
			(soildat(*,*,1) lt 9) and $
			(soildat(*,*,1) gt 0) and $
			(soildat(*,*,1) lt 8) and $
			(soildat(*,*,1) gt 83) and $
			(soildat(*,*,2) lt 100), coarse_c)
	if coarse_c gt 0 then begin
		textur(coarse)=1
		tex_fuzzy(coarse)=0.2
	endif
;----------------------------------------------------------
coars_med=WHERE((soildat(*,*,0) gt 9) and $
			(soildat(*,*,0) lt 20) and $
			(soildat(*,*,1) gt 8) and $
			(soildat(*,*,1) lt 20) and $
			(soildat(*,*,2) gt 63) and $
			(soildat(*,*,2) lt 83), coars_med_c)
	if coars_med_c gt 0 then begin
		textur(coars_med)=2
		tex_fuzzy(coars_med)=0.3
	endif
;----------------------------------------------------------
medium=WHERE((soildat(*,*,0) gt 20) and $
			(soildat(*,*,0) lt 30) and $
			(soildat(*,*,1) gt 20) and $
			(soildat(*,*,1) lt 33) and $
			(soildat(*,*,2) gt 37) and $
			(soildat(*,*,2) lt 63), medium_c)
	if medium_c gt 0 then begin
		textur(medium)=3
		tex_fuzzy(medium)=0.4
	endif
;----------------------------------------------------------
med_fine=WHERE((soildat(*,*,0) gt 30) and $
			(soildat(*,*,0) lt 48) and $
			(soildat(*,*,1) gt 25) and $
			(soildat(*,*,1) lt 33) and $
			(soildat(*,*,2) gt 27) and $
			(soildat(*,*,2) lt 37), med_fine_c)
	if med_fine_c gt 0 then begin
		textur(med_fine)=4
		tex_fuzzy(med_fine)=0.5
	endif
;----------------------------------------------------------
fine=WHERE((soildat(*,*,0) gt 48) and $
			(soildat(*,*,0) lt 67) and $
			(soildat(*,*,1) gt 17) and $
			(soildat(*,*,1) lt 25) and $
			(soildat(*,*,2) gt 17) and $
			(soildat(*,*,2) lt 27), fine_c)
	if fine_c gt 0 then begin
		textur(fine)=5
		tex_fuzzy(fine)=0.6
	endif
;----------------------------------------------------------
no_data=WHERE((soildat(*,*,*) eq -1), no_data_c)
	if no_data_c gt 0 then begin
		textur(no_data)=0
		tex_fuzzy(no_data)=0
	endif
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Bestimmung der Bodennhrstoffe
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
a=	-0.004		; Die Parameter a,b und c sind empirisch
b=	0.0971		; fr die untenangegebene Gleichung bestimmt
c=	0.4
nutri_fuzzy=FLTARR(ns,nl)
nutri_fuzzy(*,*)=a*(soildat(*,*,3)*soildat(*,*,3))+b*soildat(*,*,3)+c
	t1=WHERE(nutri_fuzzy(*,*) lt 0,count_t1)
	if (count_t1 gt 0) then nutri_fuzzy(t1)=0
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Kombination von Textur und Bodennhrstoffen
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
FUZZY_OR, tex_fuzzy, nutri_fuzzy, SOIL_fuzzy
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ende der Routine E_MAX_SOIL.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; *********************************************************
; FUZZY_OR.pro
; *********************************************************
PRO FUZZY_OR, Var1, Var2, Erg
ns=N_ELEMENTS(Var1(*,0))
nl=N_ELEMENTS(Var1(0,*))
Erg=FLTARR(ns,nl)
hlp=FLTARR(2)
FOR y=0,nl-1 DO BEGIN
 FOR x=0,ns-1 DO BEGIN
	hlp(0)=Var1(x,y)
	hlp(1)=VAR2(x,y)
	Erg(x,y)=MAX(hlp)
 ENDFOR
ENDFOR
END
; *********************************************************
; FUZZY_AND.pro
; *********************************************************
PRO FUZZY_AND, Var1, Var2, Erg
ns=N_ELEMENTS(Var1(*,0))
nl=N_ELEMENTS(Var1(0,*))
Erg=FLTARR(ns,nl)
hlp=FLTARR(2)
FOR y=0,nl-1 DO BEGIN
 FOR x=0,ns-1 DO BEGIN
	hlp(0)=Var1(x,y)
	hlp(1)=VAR2(x,y)
	Erg(x,y)=MIN(hlp)
 ENDFOR
ENDFOR
END
; *********************************************************
; NDVI_FPAR.pro
; *********************************************************
PRO NDVI_FPAR, MOD09_PFD, NDVI_PFD, FPAR_PFD, FPAR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ffnen der Mod09-Datei und Auslesen allgemeine Angaben
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, MOD09_PFD, r_fid=mod09_id
ENVI_FILE_QUERY, mod09_id, ns=ns, nl=nl, nb=nb, data_type=dt
map_info = ENVI_GET_MAP_INFO(fid=mod09_id)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung des NDVI
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
datai=INTARR(ns,nl,2)	; Erzeugen eines Dataarrays
ndvi=FLTARR(ns,nl)		; Erzeugen eines Ausgabearrays
datao=FLTARR(ns,nl,2)
dims = [-1, 0, ns-1, 0, nl-1]
datai(*,*,0) = ENVI_GET_DATA(fid=mod09_id, dims=dims, pos=0)
datai(*,*,1) = ENVI_GET_DATA(fid=mod09_id, dims=dims, pos=1)
ENVI_FILE_MNG, id=mod09_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Kalibrierung der MODIS MOD09 Spektraldaten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
mod09_cal=0.002
datao(*,*,*) = FLOAT(datai(*,*,*)) * mod09_cal
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung: NDVI=(NIR - RED)/(NIR + RED)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ndvi=(datao(*,*,1) - datao(*,*,0))/(datao(*,*,1) + datao(*,*,0))
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe der NDVI Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bnames=['NDVI']
OPENW,1,NDVI_PFD
WRITEU,1, ndvi
CLOSE,1
ENVI_SETUP_HEAD, fname=NDVI_PFD, $
		ns=ns, nl=nl, nb=1, $
 		interleave=0, data_type=4, $
 		offset=0, /write, map_info=map_info,$
 		r_fid=NDVI_id, bnames=bnames
ENVI_FILE_MNG, id=NDVI_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der FPAR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
a = 1.67		; Kalibrierung nach Seaquist et al .2003
b = 0.07		; fr westafrikanische Savanne
FPAR=FLTARR(ns,nl)
FPAR= a * (ndvi) - b
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe von FPAR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bnames=['FPAR']
OPENW,1,FPAR_PFD
WRITEU,1, FPAR
CLOSE,1
ENVI_SETUP_HEAD, fname=FPAR_PFD, $
		ns=ns, nl=nl, nb=1, $
 		interleave=0, data_type=4, $
 		offset=0, /write, map_info=map_info,$
 		r_fid=FPAR_id, bnames=bnames
ENVI_FILE_MNG, id=FPAR_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ende der Routine NDVI_FPAR.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; *********************************************************
; BILANZ.pro
; Diese Routine berechnet die aktuelle Bilanz des
; Biomassemodells und schreibt die Daten in die Ursprungs-
; datei zurck.
; Dabei werden die unterschiedlichen Vegetationslayer
; bercksichigt.
PRO BILANZ, home, date, bil_param
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; bergabe der Parameter und Ableitung der Ausgabepfade
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Angabe des Prozessierungsjahres
jahr=STRTRIM(date(0),2)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Angabe des 10-Tages-Komposits
IF date(1) LT 10 THEN step = '0'+STRTRIM(date(1),2) $
ELSE step=STRTRIM(date(1),2)
; ---------------------------------------------------------
; Erstellen der Pfade
; ---------------------------------------------------------
BIO_BALANCE_PFD=home+'\balance\BIOMASS_ALL.data'
CASE bil_param OF
'bio':DATA_PFD=home+'\'+jahr+'\tsp\biomass\TSP_BIOMASS_'+ $
			step+jahr+'.data'
'tier':DATA_PFD=home+'\'+jahr+'\tsp\grasing\TSP_CATTLE_'+ $
			step+jahr+'.data'
'wild':DATA_PFD=home+'\'+jahr+'\tsp\grasing\TSP_GAME_'+ $
			step+jahr+'.data'
ELSE:return
ENDCASE
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der NPP-Bilanz
; ---------------------------------------------------------
; Einlesen der Bilanzdatei
; ---------------------------------------------------------
ENVI_OPEN_FILE, BIO_BALANCE_PFD, r_fid=biobil_id
ENVI_FILE_QUERY, biobil_id, ns=ns, nl=nl, nb=nb, data_type=dt
map_info = ENVI_GET_MAP_INFO(fid=biobil_id)
biomass_bil=FLTARR(ns,nl,nb)
bdims = [-1, 0, ns-1, 0, nl-1]
FOR i=0,nb-1 DO BEGIN
biomass_bil(*,*,i) = ENVI_GET_DATA(fid=biobil_id, dims=bdims, pos=i)
ENDFOR
ENVI_FILE_MNG, id=biobil_id, /remove
; ---------------------------------------------------------
; Einlesen der Datendatei
; ---------------------------------------------------------
ENVI_OPEN_FILE, DATA_PFD, r_fid=data_id
ENVI_FILE_QUERY, data_id, ns=ns, nl=nl, nb=nb, data_type=dt
map_info = ENVI_GET_MAP_INFO(fid=data_id)
bil_data=FLTARR(ns,nl,nb)
ddims = [-1, 0, ns-1, 0, nl-1]
FOR i=0,nb-1 DO BEGIN
bil_data(*,*,i) = ENVI_GET_DATA(fid=data_id, dims=ddims, pos=i)
ENDFOR
ENVI_FILE_MNG, id=data_id, /remove
; ---------------------------------------------------------
; Berechnung der Bilanz
; ---------------------------------------------------------
FOR i=0,nb-1 DO BEGIN
biomass_bil(*,*,i)=biomass_bil(*,*,i)+bil_data(*,*,i)
ENDFOR
; ---------------------------------------------------------
; Ausgabe der Biomassenbilanz
; ---------------------------------------------------------
bnames=['NPP gesamt [g/m2*ts]','NPP Bare [g/m2*ts]', $
		'NPP NonTree [g/m2*ts]','NPP Tree [g/m2*ts]']
OPENW, 1, BIO_BALANCE_PFD
WRITEU, 1, biomass_bil
CLOSE, 1
ENVI_SETUP_HEAD, fname=BIO_BALANCE_PFD, $
  ns=ns, nl=nl, nb=nb, $
  interleave=0, data_type=4, $
  offset=0, /write, map_info=map_info,$
  r_fid=biobil_id, bnames=bnames
ENVI_FILE_MNG, id=biobil_id, /remove
END
; *********************************************************
; BIOMASS_SINGLE.pro
; *********************************************************
; Diese Routine wurde von J. Richters am 21.07.2003
; im Rahmen des SFB 389 - ACACIA, TP B7 am GIUB erstellt.
; *********************************************************
; J. Richters, 07/2003, RSRG, GIUB, University of Bonn
; Diese Procedur dient der Berechnung der Biomasse zu einem
; Zeitpunkt Tx der vom Zeitpunkt Tx +1 bis Tx +n gefolgt
; wird. Der erneute Aufruf der Procedur erfolgt durch eine
; bergeordnete Steuerprocedur.
; *********************************************************
PRO BIOMASS_SINGLE, home, date
; *********************************************************
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; bergabe der Parameter und Ableitung der Ausgabepfade
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Angabe des Prozessierungsjahres
jahr=STRTRIM(date(0),2)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Angabe des 10-Tages-Komposits
IF date(1) LT 10 THEN step = '0'+STRTRIM(date(1),2) $
ELSE step=STRTRIM(date(1),2)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ableitung der Eingabepfade
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
MOD09_PFD=	home+'\'+jahr+'\tsp\mod09\TSP_MOD09_'+step+ $
			jahr+'.data'
CLD_PFD =	home+'\'+jahr+'\tsp\cloud\TSP_CLD_'+step+ $
			jahr+'.data'
TMP_PFD =	home+'\'+jahr+'\tsp\temperatur\TSP_TEMP_'+ $
			step+jahr+'.data'
EMS_PFD =	home+'\'+jahr+'\tsp\emissivity\TSP_EMS_'+ $
			step+jahr+'.data'
WV_PFD =	home+'\'+jahr+'\tsp\watervapor\TSP_WV_'+step $
			+jahr+'.data'
DGM_PFD =	home+'\'+jahr+'\ysp\CDP_DGM_'+jahr+'.data'

BCF_PFD =	home+'\'+jahr+'\ysp\YSP_BCF_'+jahr+'.data'

BIO_BALANCE_PFD=home+'\balance\BIOMASS_ALL.data'
VCF_PFD =	home+'\'+jahr+'\ysp\YDP_VCF_'+jahr+'.data'
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ableitung der Ausgabepfade
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
NDVI_PFD=	home+'\'+jahr+'\tsp\ndvi\TSP_NDVI_'+step+jahr $
			+'.data'
FPAR_PFD=	home+'\'+jahr+'\tsp\fpar\TSP_FPAR_'+step+jahr $
			+'.data'
PAR_PFD=	home+'\'+jahr+'\tsp\par\TSP_PAR_'+step+jahr+ $
			'.data'
PAR_Eff_PFD=home+'\'+jahr+'\tsp\par_eff\TSP_PAR_Eff_'+ $
			step+jahr+'.data'
BIOMASS_PFD=home+'\'+jahr+'\tsp\biomass\TSP_BIOMASS_'+ $
			step+jahr+'.data'
PET_PFD =	home+'\'+jahr+'\tsp\pet\TSP_PET_'+ $
			step+jahr+'.data'
BCF_E_PFD =	home+'\'+jahr+'\tsp\BCF_Eff\TSP_BCF_e_'+ $
			step+jahr+'.data'
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der PAR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PAR, DGM_PFD, CLD_PFD, PAR_PFD, PAR_Eff_PFD, date, PAR_Eff
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der FPAR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
NDVI_FPAR, MOD09_PFD, NDVI_PFD, FPAR_PFD, FPAR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung des biophysikalischen Konversionsfaktors
; Epsilon
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
EPSILON_EFF, BCF_PFD, TMP_PFD, WV_PFD, bcf_eff, $
				 MOD09_PFD, PAR_PFD, EMS_PFD, $
				 NDVI_PFD, PET_PFD, BCF_E_PFD
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Erfassung allgemeiner Dateiparameter
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, MOD09_PFD, r_fid=mod09_id
ENVI_FILE_QUERY, mod09_id, ns=ns, nl=nl, nb=nb, data_type=dt
map_info = ENVI_GET_MAP_INFO(fid=mod09_id)
ENVI_FILE_MNG, id=mod09_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechung der Biomasse
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
npp_out=FLTARR(ns,nl,4)
npp_tsp = FLTARR(ns,nl)
npp_tsp = FPAR * PAR_Eff * bcf_eff
npp_out(*,*,0) = npp_tsp
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Lschen der Variablen
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
i_ndvi=0
PAR_Eff=0
E_Eff=0
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Aufteilen der NPP in Funktionseinheiten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, VCF_PFD, r_fid=vcf_id
ENVI_FILE_QUERY, vcf_id, ns=ns, nl=nl, nb=nb, data_type=dt
map_info = ENVI_GET_MAP_INFO(fid=vcf_id)
vcf=BYTARR(ns,nl,3)
dims = [-1, 0, ns-1, 0, nl-1]
vcf(*,*,0) = ENVI_GET_DATA(fid=vcf_id, dims=dims, pos=0)
vcf(*,*,1) = ENVI_GET_DATA(fid=vcf_id, dims=dims, pos=1)
vcf(*,*,2) = ENVI_GET_DATA(fid=vcf_id, dims=dims, pos=2)
ENVI_FILE_MNG, id=vcf_id, /remove
npp_out(*,*,0) = npp_tsp
npp_cent = npp_tsp * 0.01
npp_out(*,*,1) = npp_cent * FLOAT(vcf(*,*,0))
npp_out(*,*,2) = npp_cent * FLOAT(vcf(*,*,1))
npp_out(*,*,3) = npp_cent * FLOAT(vcf(*,*,2))
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe des NPP_TSP
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bnames=['NPP gesamt [g/m2*ts]','NPP Bare [g/m2*ts]', $
		'NPP NonTree [g/m2*ts]','NPP Tree [g/m2*ts]']
OPENW, 1, BIOMASS_PFD
WRITEU, 1, npp_out
CLOSE, 1
ENVI_SETUP_HEAD, fname=BIOMASS_PFD, $
  ns=ns, nl=nl, nb=4, $
  interleave=0, data_type=4, $
  offset=0, /write, map_info=map_info,$
  r_fid=biomass_id, bnames=bnames
ENVI_FILE_MNG, id=biomass_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ende der Routine BIOMASS_SINGLE.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; *********************************************************
; PAR.pro
; *********************************************************
; Diese Routine wurde von J. Richters
; am 21.07.2003
; im Rahmen des SFB 389 - ACACIA, Teilprojekt B7 am GIUB
; erstellt.
; *********************************************************
; Diese Procedur dient der Berechung der photosynthetisch
; aktiven Strahlung, die mit dem Chlorophyll der grnen
; Vegetation interagiert und so Biomasse bildet.
; Dabei wird die Procedur von der bergeordneten Berechung
; der NPP aufgerufen und ruft ihrerseits die Ermittlung der
; Potentiellen Solaren Einstrahlung auf.
; *********************************************************
PRO PAR, DGM_PFD, CLD_PFD, PAR_PFD, PAR_Eff_PFD, $
		 date, PAR_Eff
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; bergabe der Parameter
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jahr=STRTRIM(date(0),2) ; Angabe des Prozessierungsjahres
step=STRTRIM(date(1),2) ; Angabe des 10-Tages-Komposits
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Allgemeine Angaben
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
par_frac = 0.48			; Anteil der photosynthetisch
						; nutzbaren Strahlung (etwa 50%)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der potentiellen Solaren Einstrahlung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
RBM_INSOL, DGM_PFD, date(1), PAR_PFD, potinsol_sum
PAR_Eff = FLTARR(N_ELEMENTS(potinsol_sum(*,0)), $
		  N_ELEMENTS(potinsol_sum(0,*)))
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Integration der Wolkenmaske
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, CLD_PFD, r_fid=cld_id
ENVI_FILE_QUERY, cld_id, ns=ns, nl=nl, nb=nb, data_type=dt
map_info = ENVI_GET_MAP_INFO(fid=cld_id)
cld_mask=BYTARR(ns,nl)
dims = [-1, 0, ns-1, 0, nl-1]
cld_mask(*,*)=ENVI_GET_DATA(fid=cld_id, dims=dims, pos=0)
ENVI_FILE_MNG, id=cld_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der Extinktion der Atmosphre
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
cld_mask = FLOAT(cld_mask) / 100
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der PAR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PAR_Eff(*,*)=potinsol_sum(*,*) * par_frac * cld_mask(*,*)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe der Effektiven PAR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bnames=['Effektive Einstrahlung [MJ/m2/ts]']
OPENW, 1, PAR_Eff_PFD
WRITEU, 1, PAR_Eff
CLOSE, 1
ENVI_SETUP_HEAD, fname=PAR_Eff_PFD, $
  ns=ns, nl=nl, nb=1, $
  interleave=0, data_type=4, $
  offset=0, /write, map_info=map_info,$
  r_fid=pareff_id, bnames=bnames
ENVI_FILE_MNG, id=pareff_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine PAR.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; *********************************************************
; Dieses Program dient der Berechung der potentiellen
; Solaren Einstrahlung auf einer beliebigen Flche. Dazu
; werden lediglich drei Eingabevariablen gebraucht:
; - Julianisches Datum (Tag des Jahres, DOY)
; - Neigung der jeweiligen Rasterzelle (Slope)
; - Exposition der jeweiligen Rasterzelle (Aspect)
; *********************************************************
; Benutzung:
; das Programm gliedert sich in zwei Teile. Eine
; bergeordnete Routine liest das digitale Gelndemodell
; aus und berechnet daraus slope und aspect und erhebt die
; Angabe des Tages. Danach werden diese Angaben einem
; Punktmodell zugefhrt, das die Potentielle Solare
; Einstrahlung fr die jeweiligen Daten berchnet.
; Diese Routine stammt aus einem Aufsatz von Swift(1976):
; Algerithm for Solar Radiation on Moutain Slopes.
; In: Water Resources Research, Vol.12, No.1.
; *******************************************************************
; Diese Routine wurde von J. Richters
; am 25.02.2003 im Rahmen des SFB 389 - ACACIA,
; Teilprojekt B7 am GIUB erstellt.
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PRO INSOL_POINT, doy, A, I, L0, R4
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Eingang der Berechnung (nach Swift, 1976)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Eingabevariablen:
;doy = 174 	; doy = Day of Year nach dem julianischen
			; Kalender
;A = 270 	; A = exposition des Hanges (Aspect) in 
;I = 30		; Neigung des Hanges (Slope) in 
;L0 = 30		; Geograpgische Breite des Hanges in 
R0 = 1.95	; Solarkonstante [cal/cm 2/min-1]
; unvernderliche Variablen
pi = 3.1415927	; Pi
; Umrechung der Eingangsvariablen
A = pi * A / 180
I = pi * I / 180
L0 = pi * L0 / 180
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der Sonnenposition
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der Deklination
D = 0.007 - 0.4067 * cos((doy + 10)* 0.0172)
; Berechnung des radius Vektors der Sonne
E = 1.0 - 0.0167 * cos((doy-3) * 0.0172)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der Reliefparameter L1 und L2
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechung der quivalenzwinkel L1
L1 = asin(cos(I) * sin(L0) + sin(I) * cos(L0) * cos(A))
; Berechung der quivalenzwinkel L2
D1 = cos(I) * cos(L0) - sin(L0)*cos(A)
if (D1 eq 0) then begin
	D1 = 1.0 * E - 10
	L2 = atan((sin(I) * sin(A))/(D1))
	endif else begin
	L2 = atan((sin(I) * sin(A))/(D1))
endelse
if (D1 lt 0) then L2 = L2 + 180
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der Solarkonstante fr 60 Min
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
R1 = 60 * R0/(E*E)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der Stundenwinkel fr Sonnenscheindauer
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
T = acos(-tan(L1)*tan(D))
T7 = T - L2
T6 = -T - L2
T = acos(-tan(L0)*tan(D))
T1 = T
T0 = -T
if (T7 lt T1) then T3 = T7 else T3 = T1
if (T6 lt T0) then T2 = T6 else T2 = T0
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der potentiellen Solaren Einstrahlung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (T3 lt T2) then begin
	T2 = 0
	T3 = 0
endif
T6 = T6 + 360
if (T6 lt T1) then begin
	T6 = T6
	T9 = T1
	R4 = (R1 * (sin(D)* sin(L1) * (T3 - T2)/15 $
		+ cos(D)* cos(L1)* (sin(T3 + L2)- sin(T2 + L2))* 12/pi))$
		+ (R1 * (sin(D)* sin(L1) * (T9 - T6)/15 $
		+ cos(D)* cos(L1)* (sin(T9 + L2)- sin(T6 + L2))* 12/pi))
endif else begin
	T7 = T7 - 360
	if (T7 gt T0) then begin
	T6 = T0
	T9 = T7
	R4 = (R1 * (sin(D)* sin(L1) * (T3 - T2)/15 $
		+ cos(D)* cos(L1)* (sin(T3 + L2)- sin(T2 + L2))* 12/pi))$
		+ (R1 * (sin(D)* sin(L1) * (T9 - T6)/15 $
		+ cos(D)* cos(L1)* (sin(T9 + L2)- sin(T6 + L2))* 12/pi))
	endif else begin
	R4 = R1 * (sin(D)* sin(L1) * (T3 - T2)/15 $
		+ cos(D)* cos(L1)* (sin(T3 + L2)- sin(T2 + L2))* 12/pi)
	endelse
endelse
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der Einstrahlungsdauer
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
T4 = T2 / 15
T5 = T3 / 15
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine INSOL_POINT.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; *********************************************************
;pro, RBM_insol, doy, DGM,doy
PRO RBM_INSOL, DGM_PFD, step, PAR_PFD, potinsol_sum
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen des DGM und Berechung der Reliefparameter (Slope & Aspect)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, DGM_PFD,/no_realize, r_fid=fid
ENVI_FILE_QUERY, fid, ns=ns, nl=nl
proj = ENVI_GET_PROJECTION(fid=fid, pixel_size=pixel_size)
dims = [-1l, 0, ns-1, 0, nl-1]
pos  = [0]
bptr = [0,1]
ENVI_DOIT, 'topo_doit', $
    fid=fid, pos=pos, dims=dims, $
    bptr=bptr, /in_memory , $
    pixel_size=pixel_size, r_fid=r_fid
data=FLTARR(ns,nl,4)
data(*,*,0) = ENVI_GET_DATA(fid=fid, dims=dims, pos=0)
data(*,*,1) = ENVI_GET_DATA(fid=r_fid, dims=dims, pos=0)
data(*,*,2) = ENVI_GET_DATA(fid=r_fid, dims=dims, pos=1)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der Latitude fr jeder Zeile des DGM
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Umprojezieren der Daten um die Geographische Breite
; zu finden. Einlesen der vorhanden Map_info und Aus-
; lesen von Projektionparametern
map_info = ENVI_GET_MAP_INFO(fid=fid)
ps_y=map_info.ps(1)
iymap=map_info.mc(3)
ixmap=map_info.mc(2)
; Uebernahme der Eingangsprojektion aus Map Info und
; Erzeugen einer neuen Projektion
iproj = map_info.proj
oproj = ENVI_PROJ_CREATE(/geographic)
; Oeffnung einer Schleife um Geogr. Breite fr jeder
; Zeile zu berechnen. Die Breitenangabe wird in der
; Ebene 4 des Array Data(*,*,3) gespeichert.
for i=0,nl-1 do begin
	ENVI_CONVERT_PROJECTION_COORDINATES, ixmap, iymap, $
			iproj, oxmap, oymap, oproj
	iymap=iymap - ps_y
	data(*,i,3)=oymap
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Schlieen und Entfernen der offenen Dateien
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_FILE_MNG, id=fid, /remove ;,/delete
ENVI_FILE_MNG, id=r_fid, /remove, /delete
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Eingabe des Tages (des Jahres: 1-365)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
doy_o = step * 10
doy_a=doy_o - 10
if (doy_a lt 1) then doy_a = 1
if (doy_o gt 365) then doy_o = 365
nod=10	; Anzahl der Tage pro Rechenschritt
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der Potentiellen Einstrahlung
; mit Hilfe der Subroutine 'insol_point.pro'
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabearray der aufsummierten Insol-Daten
potinsol_sum=FLTARR(ns,nl)
doy=doy_a
for j=0,10-1 do begin
	; Ausgabearray der berechneten Daten
	potinsol=FLTARR(ns,nl)
	; Schleife zur Adressierung jeder Rasterzelle
	for y=0, nl-1 do begin
		for x=0, ns-1 do begin
		A = data(x,y,2)
		I = data(x,y,1)
		L0 = data(x,y,3)
		R4sum = 0. 	;Aufsummierte Einstrahlung
		INSOL_POINT, doy, A, I, L0, R4
		potinsol(x,y)=R4
		endfor
	endfor
potinsol_sum(*,*)=potinsol_sum(*,*) + potinsol(*,*)
doy=doy+1
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Umrechnung der Energie von Cal in J und pro qm
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Cal_to_J = 4.186			; 1 cal = 4.186 J
potinsol_sum = (potinsol_sum * Cal_to_J * 10000) / 1000000
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe der Pot. Einstrahlung in eine Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bnames=['Potentielle Einstrahlung [MJ/m2/ts]']
OPENW,1,PAR_PFD
WRITEU,1,potinsol_sum
CLOSE, 1
ENVI_SETUP_HEAD, fname=PAR_PFD, $
  ns=ns, nl=nl, nb=1,  interleave=0,$
  data_type=4, offset=0, /write, $
  map_info=map_info, bnames=bnames, $
  r_fid=pot_sol_id
ENVI_FILE_MNG, id=pot_sol_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ende der Routine RBM_INSOL.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; *********************************************************
; EPSILON_EFF.pro
; *********************************************************
; Die Routine EPSILON_EFF.pro berechnet den effektiven
; biophysikalischen Konversionfaktor (BCF) epsilon unter
; Einbeziehung folgender Datenstze:
; - YSP_BCF_YYYY.data			(Epsilon_Max)
; - TSP_WV_XXYYYY.data			(MODIS Wasserdampf)
; - TSP_TMP_XXYYYY.data			(MODIS Surface Temperatur)
; - TSP_EMS_XXYYYY.data			(MODIS Surface Emissivity)
; - TSP_INSOL_XXYYYY.data		(Solare Einstrahlung)
; *********************************************************
; Diese Routine wurde von J. Richters
; am 15.12.2003 im Rahmen des SFB 389 - ACACIA,
; Teilprojekt B7 am GIUB erstellt.
; *********************************************************
PRO EPSILON_EFF, BCF_PFD, TMP_PFD, WV_PFD, bcf_eff, $
				 MOD09_PFD, PAR_PFD, EMS_PFD, NDVI_PFD, $
				 PET_PFD, BCF_E_PFD
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Erfassung des Temperaturstesses
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, TMP_PFD, r_fid=tmp_id, /no_realize
ENVI_FILE_QUERY, tmp_id, ns=ns, nl=nl
map_info = ENVI_GET_MAP_INFO(fid=tmp_id)
temp = FLTARR(ns,nl)
dims = [-1, 0, ns-1, 0, nl-1]
temp(*,*) = ENVI_GET_DATA(fid=tmp_id, dims=dims, pos=0)
ENVI_FILE_MNG, id=tmp_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung des Temperaturstess (Skaliert von 0-1)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
T_stress = 0.8 + 0.02 * (temp) - 0.0005 * (temp^2)
; Eingrenzung auf gltigen Wertebereich
t_hilf=WHERE(T_stress lt 0, t_count)
if t_count gt 0 then T_stress(t_hilf)=0
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Erfassung des Wasserstress mit akt. und pot. Veerdunstung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung der PET
; ---------------------------------------------------------
PET_all, TMP_PFD, PAR_PFD, EMS_PFD, MOD09_PFD, PET, $
		 NDVI_PFD, PET_PFD
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der aktuellen Verdunstung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, WV_PFD, r_fid=wv_id, /no_realize
ENVI_FILE_QUERY, wv_id, ns=ns, nl=nl
wv = FLTARR(ns,nl)
dims = [-1, 0, ns-1, 0, nl-1]
wv(*,*) = ENVI_GET_DATA(fid=wv_id, dims=dims, pos=0)
ENVI_FILE_MNG, id=wv_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;Berechnung des pflanzenrelevanten Wasserdampfs (ca. 10%)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wv = wv * 0.1
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;Berechnung des Wasserstess (Skaliert von 0-1)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
W_stress = 0.5 + 0.5 * (wv/PET)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Eingrenzung auf gltigen Wertebereich
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
w_hilf=WHERE(FINITE(W_stress) eq 0, w_count)
if w_count gt 0 then W_stress(W_hilf) = 0
w_hilf=WHERE(W_stress gt 1.0, w_count)
if w_count gt 0 then W_stress(W_hilf) = 1
w_hilf=WHERE(W_stress lt 0, w_count)
if w_count gt 0 then W_stress(W_hilf) = 0
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;Ausgabe des Wasser- & Temperaturstess (Skaliert von 0-1)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PF_stress=FLTARR(ns,nl,2)
PF_stress(*,*,0)=T_stress(*,*)
PF_stress(*,*,1)=W_stress(*,*)
PF_stress_out=BCF_E_PFD+'_PF_stress.data'
OPENW,1,PF_stress_out
WRITEU,1,PF_stress
CLOSE, 1
bnames=['T_Stress','W_Stress']
ENVI_SETUP_HEAD, fname=PF_stress_out, $
  ns=ns, nl=nl, nb=2,  interleave=0,$
  data_type=4, offset=0, /write, $
  map_info=map_info, bnames=bnames, $
  r_fid=bcf_e_id
ENVI_FILE_MNG, id=bcf_e_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechnung des effekitven biophysikalischen
; Konversionsfaktors (BCF_eff))
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, BCF_PFD, r_fid=bcf_id, /no_realize
ENVI_FILE_QUERY, bcf_id, ns=ns, nl=nl
bcf_max = FLTARR(ns,nl)
dims = [-1, 0, ns-1, 0, nl-1]
bcf_max(*,*) = ENVI_GET_DATA(fid=bcf_id, dims=dims, pos=0)
ENVI_FILE_MNG, id=bcf_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Kombination der Stressfaktoren
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bcf_eff = FLTARR(ns,nl)
bcf_eff = bcf_max * ((T_stress + W_stress)/2)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe der BCF-Effektiv-Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bnames=['eff. BCF [MJ/g C]']
OPENW,1,BCF_E_PFD
WRITEU,1,bcf_eff
CLOSE, 1
ENVI_SETUP_HEAD, fname=BCF_E_PFD, $
  ns=ns, nl=nl, nb=1,  interleave=0,$
  data_type=4, offset=0, /write, $
  map_info=map_info, bnames=bnames, $
  r_fid=bcf_e_id
ENVI_FILE_MNG, id=bcf_e_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine EPSILON_EFF.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; #########################################################
; In den Modul PET wird mit Hilfe der Priestley-Taylor-
; Gleichung die potentielle Verdunstung fr jeden Punkt
; im Modellraster berechnet.
; Dabei wird die Routine PET_single.pro aufgerufen, die
; die PET punkthaft berechnet. Die bergeordnetr Routine
; PET_all.pro steuert die Routine an und liefert die
; bentigten Eingangsparameter.
; #########################################################
PRO PET_SINGLE, T, Es, EMS, alb, PET, NDVI
; Dieses Modell dienst der Ermittlung der Potentiellen
; Evaporation dabei wird der Ansatz von Penman-Monteith
; und Priestley-Taylor genutzt.
; ---------------------------------------------------------
; bernahme von Punktdaten aus bergeordneter Routine
; ---------------------------------------------------------
;T = 30			; Temperatur [C]
;Es = 			; GLobalstrahlung [MJ/m2] aus
				; dem Modell von Swift(1976)
;Ems = 			; Emissivitt [MJ/m2] aus MOD11
;alb = 			; Albedo
; ---------------------------------------------------------
; Konstante Parameter
; ---------------------------------------------------------
a = 1.26			; Priestley-Taylor-Parameter
					; (Advektiver Parameter)
gamma = 0.667		; psychrometrische Konatante [kPa/C]
sigma = 5.67*10^(-8); Stefan-Bolzmann-Konstante [W m2 K4]

; ---------------------------------------------------------
; Variable Parameter (inkl. Berechnung)
; ---------------------------------------------------------
delta = 6.1078^((7.5 * T)/(237.3 + T))		; Sttigungs-
					; dampfdruck (fr T > 0) nach Weischet (1995)
Rn = (1- alb)*Es - EMS * sigma*T^4

G = (0.583^(-2.13 * NDVI)) * Rn	; Bodenwrmestrom [MJ/m2]

; ---------------------------------------------------------
; Berechnung der Energie fr pot. Evapotranspiration
; ---------------------------------------------------------
lambdaE = a*(delta/(delta + gamma)) * (Rn - G)	;[MJ/m2]
; ---------------------------------------------------------
; Berechnung der pot. Evapotranspiration
; ---------------------------------------------------------
PET = lambdaE / 2450	; Pot. Verdunstungsrate [kg/m2]
; ---------------------------------------------------------
; Anpassung der Einheiten von kg/m2 auf g/m2
; ---------------------------------------------------------
PET = PET * 100	; Pot. Verdunstungsrate [g/m2]=[cm/m2]
; ---------------------------------------------------------
; Ende der Routine PET_single.pro
; ---------------------------------------------------------
END
; #########################################################
; #########################################################
PRO PET_all, TMP_PFD, INSOL_PFD, EMS_PFD, MOD09_PFD, $
			PotEvapo , NDVI_PFD, PET_PFD
; ---------------------------------------------------------
; Einlesen der Dateien um PET abzuleiten
; ---------------------------------------------------------
; Einlesen der Temperaturdaten
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ENVI_OPEN_FILE, TMP_PFD, r_fid=tmp_id, /no_realize
ENVI_FILE_QUERY, tmp_id, ns=ns, nl=nl
map_info = ENVI_GET_MAP_INFO(fid=tmp_id)
temp = FLTARR(ns,nl)
dims = [-1, 0, ns-1, 0, nl-1]
temp(*,*) = ENVI_GET_DATA(fid=tmp_id, dims=dims, pos=0)
ENVI_FILE_MNG, id=tmp_id, /remove
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Einlesen der Eff. Einstrahlung
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ENVI_OPEN_FILE, INSOL_PFD, r_fid = insol_id, /no_realize
insol = FLTARR(ns,nl)
insol(*,*) = ENVI_GET_DATA(fid=insol_id, dims=dims, pos=0)
ENVI_FILE_MNG, id=insol_id, /remove
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Einlesen der Emissivitt
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ENVI_OPEN_FILE, EMS_PFD, r_fid = EMS_id, /no_realize
emiss = FLTARR(ns,nl)
emiss(*,*) = ENVI_GET_DATA(fid=EMS_id, dims=dims, pos=0)
ENVI_FILE_MNG, id=EMS_id, /remove
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Einlesen der MOD09 & Generierung der Albedo
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ENVI_OPEN_FILE, MOD09_PFD, r_fid = ALB_id, /no_realize
ENVI_FILE_QUERY, ALB_id, ns=ns, nl=nl, nb=nb
alb_dum = INTARR(ns,nl,nb)
FOR i=0, 4 DO BEGIN
  alb_dum(*,*,i) = ENVI_GET_DATA(fid=ALB_id, dims=dims, pos=i)
ENDFOR
ENVI_FILE_MNG, id=ALB_id, /remove
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Kalibrierung der MODIS MOD09 Spektraldaten
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
mod09_cal=0.002
alb_dum(*,*,*) = FLOAT(alb_dum(*,*,*)) * mod09_cal
dum=FLTARR(N_ELEMENTS(alb_dum(0,0,*)))
albedo = FLTARR(ns,nl)
FOR Y=0, nl-1 DO BEGIN
 FOR X=0, ns-1 DO BEGIN
	dum = alb_dum(x,y,*)
	out=MEAN(dum)
	albedo(x,y) = out
 ENDFOR
ENDFOR
alb_dum = 0
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Einlesen des NDVI
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ENVI_OPEN_FILE, NDVI_PFD, r_fid = NDVI_id, /no_realize
NDVI = FLTARR(ns,nl)
NDVI(*,*) = ENVI_GET_DATA(fid=NDVI_id, dims=dims, pos=0)
ENVI_FILE_MNG, id=NDVI_id, /remove
; ---------------------------------------------------------
; Beginn einer Doppelschleife um PET pro Punkt zu berechnen
; ---------------------------------------------------------
PotEvapo = FLTARR(ns,nl)
FOR y=0,nl-1 DO BEGIN
 FOR x=0,ns-1 DO BEGIN
	T = temp(x,y)
	Es = insol(x,y)
	EMS = emiss(x,y)
	alb= albedo(x,y)
	ndv = ndvi(x,y)
	PET_SINGLE, T, Es, Ems, alb, PET, ndv
	PotEvapo(x,y) = PET
 ENDFOR
ENDFOR
; ---------------------------------------------------------
; Ausgabe der PET
; ---------------------------------------------------------
bnames=['Potentielle Evapotranspiration [cm]']
OPENW,1,PET_PFD
WRITEU,1,PotEvapo
CLOSE, 1
ENVI_SETUP_HEAD, fname=PET_PFD, $
  ns=ns, nl=nl, nb=1,  interleave=0,$
  data_type=4, offset=0, /write, $
  map_info=map_info, bnames=bnames, $
  r_fid=pet_id
ENVI_FILE_MNG, id=pet_id, /remove
; ---------------------------------------------------------
; Ende der Routine PET_all.pro
; ---------------------------------------------------------
END
; #########################################################
PRO READ_RBM, RBM_PFD, PARAM, Wert
; *********************************************************
; J. Richters, 07/2003, RSRG, GIUB, University of Bonn
; Diese Routine liest die Parameterdatei fr das
; RBM Kaokoveld und sucht einzelne Parameter
; heraus und gibt sie zurck.
OPENR,1, RBM_PFD
a=''
z=0
WHILE NOT EOF(1) DO BEGIN
   ; Read a line of text:
   READF, 1, A
   ;print, a
  z=z+1
ENDWHILE
CLOSE,1
b=STRARR(2,z)
OPENR,1, RBM_PFD
FOR i=0,z-1 DO BEGIN
	READF,1,A
	b(0,i)=STRMID(A,0,STRPOS(A,'=',/REVERSE_SEARCH))
	b(1,i)=STRMID(A,STRPOS(A,'=',/REVERSE_SEARCH)+1)
ENDFOR
CLOSE,1

Index=WHERE (B(0,*) eq PARAM,count)
IF count eq 1 THEN WERT=B(1,Index)
END
; #########################################################
PRO RBM_KAOKOVELD
; *********************************************************
; RBM_Steuerung
; J. Richters, 07/2003, RSRG, GIUB, University of Bonn
; Diese Routine dient der generellen Steuerung des
; RBM Kaokoveld und ruft vor allem die Routine
; RBM_Year.pro mit der nachgeordneten biomass_single.pro
; auf.
; *********************************************************
; Auswahl des Modellaufs (von RBM_Prep.pro angelegt)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;home_pfd='E:\RBM_test_01'
home=ENVI_PICKFILE(TITLE='Please select Modelling Run '+$
	 'Directory',/DIRECTORY, /OUTPUT, DEFAULT=home_pfd)
IF home eq '' THEN RETALL
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Zeitliche Eingrenzung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
MODEL_TIM=STRARR(4)
modelling_start=	[1,2001]
modelling_end=		[36,2001]
RBM_PFD=home +'\rbm_pref.rbm'
PARAM='MOD_TIME_Start_TS'
READ_RBM, RBM_PFD, PARAM, MODEL_TIM(0)
PARAM='MOD_TIME_Start_YR'
READ_RBM, RBM_PFD, PARAM, MODEL_TIM(1)
PARAM='MOD_TIME_END_TS'
READ_RBM, RBM_PFD, PARAM, MODEL_TIM(2)
PARAM='MOD_TIME_END_YR'
READ_RBM, RBM_PFD, PARAM, MODEL_TIM(3)
MODEL_TIM = FIX(MODEL_TIM)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Angabe der unvernderlichen Modellparameter
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
utm_zone=33
datum='WGS84'
TS_pa=36		; Angabe der Modellzeitschritte
				; pro Jahr (= 36)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; GUI zum Einlesen der Modellparameter
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechung der Modellaufzeit
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Wichtige Parameter zur Weitergabe innerhalb des Modells:
; MODEL_TIME      INT       = Array[year, 4]
; MODEL_TIME(year,0) = Jahr des Modelllaufs
; MODEL_TIME(year,1) = Anzahl der Modell Time Steps
; MODEL_TIME(year,2) = erster Time Step
; MODEL_TIME(year,3) = letzter Time Step
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der Modelljahre
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
modelling_years=modelling_end(1) - modelling_start(1)+1
y=0
MODEL_TIME=INTARR(modelling_years,4)
FOR t=0,modelling_years-1 DO BEGIN
  MODEL_TIME(t,0)=modelling_start(1)+y
  y=y+1
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der Modellzeitschritte
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
MODEL_TIME(0,1) = TS_pa - modelling_start(0) +1
MODEL_TIME(0,2) = modelling_start(0)
MODEL_TIME(0,3) = TS_pa
FOR t=1,modelling_years-2 DO BEGIN
  MODEL_TIME(t,1) = TS_pa
  MODEL_TIME(t,2) = 1
  MODEL_TIME(t,3) = TS_pa
ENDFOR
MODEL_TIME(modelling_years -1,1) = modelling_end(0)
MODEL_TIME(modelling_years -1,2) = 1
MODEL_TIME(modelling_years -1,3) = modelling_end(0)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Beginn der Modellberechung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
for i=0,modelling_years-1 do begin
	RBM_year, home, model_time(i,*)
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Schliessen aller Dateien und Beenden der LogFile
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ende der Routine RBM_Kaokoveld_all.pro (RBM-Steuerung)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END