; MOD05_PREP.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Diese Routine wurde im Rahmen des Teilprojekt B7 im
; SFB 389 ACACIA von J. Richters am 13.10.2003 am GIUB
; erstellt.
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Die Routinensammlung MOD0_PREP.pro dient der
; unberwachten Vorbereitung von MODIS MOD05_L2 Daten fr
; das RBM. Dabei werden die von der Hauptroutine
; MOD05_PREP.pro eingespeisten Daten verschiedenen
; Verarbeitungsschritten unterzogen. Dies sind im einzelnen:
; - MOD05_IMPORT.pro :  Hier werden die *.hdf-Daten gelesen
;						und ausgewertet. Dann werden die
;						Daten nach UTM, WGS84, Zone 33S
;						umprojeziert.
; - MOD05_PRESUB.pro :  Die importierten Daten werden
;						daraufhin grob zugescnitten, da die
;						Daten im Auslieferungszustand meist
;						erheblich zu gro sind
;						(ca. 2230 x 2270 Pixel).
; - MOD05_CALC.pro : 	In diesem Modul werden nun die
;						Daten in Ihre physikalischen
;						Ausgangsgren zurckgefhrt und
;						so abgelegt, damit sie vom RBM
;						verarbeitet werden knnen.
; - MOD05_IMPORT_TS.pro:Da fr die zu verarbeitenden Daten
;						eine Zeitreihe vorliegt, werden
;						die Daten einzeln den drei oben
;						genannten Prozessierungsschritten
;						unterzogen.
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PRO MOD05_IMPORT, name, out_wv, out_cld, ps
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ffnen der *.HDF-Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Result = EOS_QUERY( name, Info)
fid = EOS_SW_OPEN(name)
swathID = EOS_SW_ATTACH(fid, info.swath_names)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Auslesen der Datenfelder
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
nflds = EOS_SW_INQDATAFIELDS(swathID, fieldlist, rank, numbertype)
datafield=STRSPLIT(fieldlist,',',/Extract)
status = EOS_SW_READFIELD(swathID, datafield(5), cloud)
status = EOS_SW_READFIELD(swathID, datafield(7), wv_nr)
status = EOS_SW_READFIELD(swathID, datafield(6), wv_nr_corr)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Auslesen der Geoinfo
; Ermittlung der Dimension des Datenarrays
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ndims = EOS_SW_INQDIMS(swathID,dimname, dims)
dimfield=STRSPLIT(dimname,',',/Extract)
nflds = EOS_SW_INQGEOFIELDS(swathID, fieldlist, rank, numbertype)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Zerlegen des returns "fieldlist" in zwei Substrings
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
geofield=STRSPLIT(fieldlist,',',/Extract)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Auslesen der LAT & LON Datenfelder
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
status = EOS_SW_READFIELD(swathID, geofield(0), lat)
status = EOS_SW_READFIELD(swathID, geofield(1), lon)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Auslesen der EOS_SW_mapinfo
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
status = EOS_SW_MAPINFO(swathID, dimfield(2), dimfield(0), $
						lat_offset, lat_increment)
status = EOS_SW_MAPINFO(swathID, dimfield(3), dimfield(1), $
						lon_offset, lon_increment)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Beenden des HDF-Auslesens
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
status = EOS_SW_DETACH(swathID)
status = EOS_SW_CLOSE(fid)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ende der HDF Verarbeitung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Beginn der Geoverarbeitung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Generierung der Arraylnge und -breite
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
x=LINDGEN(dims(3))
y=LINDGEN(dims(2))
x_pixel=LONARR(dims(3), dims(2))
y_pixel=LONARR(dims(3), dims(2))
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der Pixelpositionen
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
for i=0, dims(2)-1 do begin
x_pixel(*,i)=lon_offset + ((x(*) * lon_increment))
endfor
for i=0, dims(3)-1 do begin
y_pixel(i,*)=lon_offset + ((y(*) * lat_increment))
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Erzeugung der Pixelvektoren
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
x_all_base=REFORM(x_pixel,1,(dims(3)*dims(2)))
y_all_base=REFORM(y_pixel,1,(dims(3)*dims(2)))
; Erzeugng der Koordinatenvektoren
ixmap_all=REFORM(lon,1,(dims(3)*dims(2)))
iymap_all=REFORM(lat,1,(dims(3)*dims(2)))
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Umprojektion der LAT/LON-Koordinaten in UTM
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
units = ENVI_TRANSLATE_PROJECTION_UNITS('Meters')
datum='WGS-84'
zone=33
iproj = ENVI_PROJ_CREATE(/geographic, datum=datum)
oproj = ENVI_PROJ_CREATE(/utm, /south, zone=zone, $
						datum=datum, units=units)
; Convert from Geographic to UTM ;
ENVI_CONVERT_PROJECTION_COORDINATES, $
  ixmap_all, iymap_all, iproj, $
  oxmap_all, oymap_all, oproj
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; bernahm der Daten aus der HDF Datei in ENVI
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_ENTER_DATA, cloud, r_fid=cld_id
ENVI_FILE_QUERY, cld_id, ns=ns, nl=nl, nb=nb, data_type=dt
CLOUD = BYTARR(ns,nl)
dims=[-1, 0, ns-1, 0, nl-1]
CLOUD = ENVI_GET_DATA(fid=cld_id, dims=dims, pos=0)
WV=INTARR(ns,nl,2)
ENVI_ENTER_DATA, wv_nr, r_fid=wv_nr_id
ENVI_ENTER_DATA, wv_nr_corr, r_fid=wv_nr_corr_id
WV(*,*,0) = ENVI_GET_DATA(fid=wv_nr_id, dims=dims, pos=0)
WV(*,*,1) = ENVI_GET_DATA(fid=wv_nr_corr_id, dims=dims, pos=0)
ENVI_FILE_MNG,id=wv_nr_id, /remove
ENVI_FILE_MNG,id=wv_nr_corr_id, /remove
ENVI_ENTER_DATA, wv, r_fid=wv_id
ENVI_FILE_QUERY, wv_id, ns=ns, nl=nl, nb=nb, data_type=dt
ENVI_FILE_MNG, id=wv_nr_id, /remove
ENVI_FILE_MNG, id=wv_nr_corr_id, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Beginn der Umprojektion der Daten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; All. Parameter fr die Umprojektion
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
pts = ([oxmap_all, oymap_all, X_all_base, Y_all_base ])
;ps = [ps, ps]
method=6 		;Triangulation with nearest neighbor
degree=1 		;
background=0	;
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Parameter fr WV
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
pos = [0,1]
bnames = ['WV Correction Factor','Total Water Vapor - NIR']
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Beginn der Image-to-Map-Regstrierung fr WV
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_DOIT, 'envi_register_doit', $
  w_fid=wv_id, w_pos=pos, w_dims=dims, $
  method=method, out_name=out_wv, $
  pts=pts, pixel_size=ps, $
  proj=oproj, OUT_BNAME=bnames, r_fid=wv_warp
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Parameter fr Cloud Mask
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
pos = 0
bnames = ['Cloud Mask']
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Beginn der Image-to-Map-Regstrierung fr WV
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_DOIT, 'envi_register_doit', $
  w_fid=cld_id, w_pos=pos, w_dims=dims, $
  method=method, out_name=out_cld, $
  pts=pts, pixel_size=ps, $
  proj=oproj, OUT_BNAME=bnames, r_fid=cld_warp
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Schlieen aller offenen Dateien
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_FILE_MNG, id=wv_warp, /remove
ENVI_FILE_MNG, id=wv_id, /remove
ENVI_FILE_MNG, id=cld_id, /remove
ENVI_FILE_MNG, id=cld_warp, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine MOD05_IMPORT
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; #########################################################
; MOD05_PRESUB.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PRO MOD05_PRESUB, data_name, xmap, ymap,ps, nns, nnl
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Konstante Parameter
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
utm_zone = 33
datum = 'WGS84'
;ps = [ps,ps]
units = ENVI_TRANSLATE_PROJECTION_UNITS('Meters')
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ffnen der Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, data_name, r_fid=fid, /no_realize
  IF (fid EQ -1) THEN RETURN
ENVI_FILE_QUERY, fid, ns=ns, nl=nl, nb=nb, data_type=dt, $
				 bnames=bnames
pos=INTARR(nb)
map_info = ENVI_GET_MAP_INFO(fid=fid)
proj = ENVI_PROJ_CREATE(/utm, zone=utm_zone, /south, $
  datum=datum, units=units)
x_map = FLTARR(2)
y_map = FLTARR(2)
x_map(0) = map_info.mc(2)
y_map(0) = map_info.mc(3)
x_map(1) = x_map(0) + (ns * ps(0))
y_map(1) = y_map(0) - (nl * ps(1))
dims = [-1l, 0, ns-1, 0, nl-1]
; ---------------------------------------------------------
; Erzeugung einer virtuellen Datei fr um daten und
; Umgebung aufzunehmen
; ---------------------------------------------------------
x_max_dim = 1500 + ns + 1500
y_max_dim = 1500 + nl + 1500
if (dt eq 1) then v_data=BYTARR(x_max_dim, Y_max_dim,nb)
if (dt eq 2) then v_data=INTARR(x_max_dim, Y_max_dim,nb)
if (dt eq 3) then v_data=LONARR(x_max_dim, Y_max_dim,nb)
if (dt eq 4) then v_data=FLTARR(x_max_dim, Y_max_dim,nb)
; ---------------------------------------------------------
; Einlesen der MOD05-Daten
; ---------------------------------------------------------
for i=0, nb-1 do begin
v_data(1500:(1500+ns-1),1500:(1500+nl-1),i)=ENVI_GET_DATA(fid=fid, dims=dims, pos=i)
endfor
envi_file_mng, id=fid, /remove ;,/delete   ;Schlieen der Data-Datei
; ---------------------------------------------------------
; Definition der map_info
; ---------------------------------------------------------
units = ENVI_TRANSLATE_PROJECTION_UNITS('Meter')
datum = 'WGS84'
vmc = [1500D, 1500, x_map(0), y_map(0)]
virtu_map_info = ENVI_MAP_INFO_CREATE(proj=proj, MC = vmc, PS = ps)
; Einladen der virtuellen Datei
ENVI_ENTER_DATA, v_data, map_info = virtu_map_info, r_fid = v_fid
; ---------------------------------------------------------
; Transformation der Map-Koordinaten in File-Pixel
; ---------------------------------------------------------
ENVI_CONVERT_FILE_COORDINATES, v_fid, xf, yf, xmap, ymap
xf=FIX(xf)
yf=FIX(yf)
print, 'SUBSET PIXEL X(round):', xf
print, 'SUBSET PIXEL Y(round):', yf
print, 'nns:',nns, 'nnl:', nnl
; ---------------------------------------------------------
; Bestimmung der Zeilen und Spalten des Subsets
; ---------------------------------------------------------
; Bestimmung der Spalten (X-Richtung)
dims = [-1, xf, (xf+nns)-1, yf, (yf+nnl)-1]
ENVI_FILE_QUERY, v_fid, nb=nb, data_type=dt
pos=INTARR(nb)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (dt eq 1) then data=BYTARR(nns,nnl,nb)
if (dt eq 2) then data=INTARR(nns,nnl,nb)
if (dt eq 3) then data=LONARR(nns,nnl,nb)
if (dt eq 4) then data=FLTARR(nns,nnl,nb)
if (dt eq 5) then data=DBLARR(nns,nnl,nb)
if (dt eq 6) then data=DCOMPLEXARR(nns,nnl,nb)
if (dt eq 14) then data=LONARR(nns,nnl,nb)
if (dt eq 12) then data=UINTARR(nns,nnl,nb)
if (dt eq 13) then data=ULONARR(nns,nnl,nb)
if (dt eq 15) then data=ULON64ARR(nns,nnl,nb)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
for i=0, nb-1 do begin
data(*,*,i)=ENVI_GET_DATA(fid=v_fid, dims=dims, pos=i)
endfor
ENVI_FILE_MNG, id=v_fid, /remove ;,/delete
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe der kopierten Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
OPENW, 1, data_name
WRITEU,1, data
CLOSE,1
mc = [0D, 0, xmap, ymap]
map_info_out = ENVI_MAP_INFO_CREATE(proj=proj, MC = mc, PS = ps)
ENVI_SETUP_HEAD, fname=data_name, $
  ns=nns, nl=nnl, nb=nb, $
  interleave=0, data_type=dt, $
  offset=0, file_type=0, bnames=bnames, $
  map_info=map_info_out, /write, r_fid=oid
envi_file_mng, id=oid, /remove ;,/delete
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine MOD05_PRESUB.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; #########################################################
PRO MOD05_CALC_WV, data_name
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ffnen der WV-Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, data_name, r_fid=fid, /no_realize
  if (fid eq -1) then RETURN
ENVI_FILE_QUERY, fid, ns=ns, nl=nl, nb=nb, data_type=dt, $
				 bnames=bnames
pos=INTARR(nb)
dims = [-1, 0, ns-1, 0, nl-1]
map_info = ENVI_GET_MAP_INFO(fid=fid)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der WV-Daten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wv_data = INTARR(ns,nl)
corr_data = INTARR(ns,nl)
wv_data = ENVI_GET_DATA(fid=fid, dims=dims, pos=1)
corr_data = ENVI_GET_DATA(fid=fid, dims=dims, pos=0)
ENVI_FILE_MNG, id=fid, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Bestimmung des Wasserdampfgehalts
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; dazu werden die Daten in ein Skalal geschrieben,
; damit sie sich leichter abfragen lassen.
data = FLTARR(ns,nl)
data(*)=0
; ---------------------------------------------------------
FOR y=0,nl-1 DO BEGIN
FOR x=0,ns-1 DO BEGIN
	IF corr_data(x,y) LT 1  THEN BEGIN
	data(x,y) = 0
	ENDIF ELSE BEGIN
	data(x,y) = FLOAT(wv_data(x,y)) / FLOAT(corr_data(x,y))
	ENDELSE
ENDFOR
ENDFOR
; ---------------------------------------------------------
data = REFORM(data, ns, nl, /OVERWRITE)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe der binren Bewlkungsdatei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
OPENW, 1, data_name
WRITEU,1, data
CLOSE,1
bnames=['Total Water Vapor - NIR [cm]']
ENVI_SETUP_HEAD, fname=data_name, $
  ns=ns, nl=nl, nb=1, $
  interleave=0, data_type=4, $
  offset=0, file_type=0, bnames=bnames, $
  map_info=map_info, /write, r_fid=oid
ENVI_FILE_MNG, id=oid, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine MOD05_CALC_WV.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; #########################################################
PRO MOD05_CALC_CLD, data_name
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ffnen der CLD-Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, data_name, r_fid=fid, /no_realize
  IF (fid EQ -1) THEN RETURN
ENVI_FILE_QUERY, fid, ns=ns, nl=nl, nb=nb, data_type=dt, $
				 bnames=bnames
pos=INTARR(nb)
dims = [-1, 0, ns-1, 0, nl-1]
map_info = ENVI_GET_MAP_INFO(fid=fid)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der CLD-Daten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
cld_data = BYTARR(ns,nl,nb)
FOR i=0,nb-1 DO BEGIN
cld_data = ENVI_GET_DATA(fid=fid, dims=dims, pos=i)
ENDFOR
ENVI_FILE_MNG, id=fid, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Bestimmung der bewlkten Pixel
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
data = BYTARR(ns,nl)
FOR y=0,nl-1 DO BEGIN
 FOR x=0,ns-1 DO BEGIN
   BIT_EXTRACT, cld_data(x,y), 1, dummy
   data(x,y) = dummy
 ENDFOR
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe der binren Bewlkungsdatei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
OPENW, 1, data_name
WRITEU,1, data
CLOSE,1
ENVI_SETUP_HEAD, fname=data_name, $
  ns=ns, nl=nl, nb=nb, $
  interleave=0, data_type=1, $
  offset=0, file_type=0, bnames=bnames, $
  map_info=map_info, /write, r_fid=oid
ENVI_FILE_MNG, id=oid, /remove ;,/delete
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine MOD05_CALC_CLD.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
;##########################################################
PRO BIT_EXTRACT, zahl, stelle, bit
dummy=BYTE(zahl/(2^stelle))
bit=dummy MOD 2
END
; #########################################################
PRO MOD05_IMPORT_SINGLE_TS, name, out_wv, out_cld,$
							xmap, ymap, ps, ns, nl
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Import der MODIS MOD05_L2 Daten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
MOD05_IMPORT, name, out_wv, out_cld, ps
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Reduktion der inportierten Daten auf Untersuchungsgebiet
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Subset der WaterVapor-Daten
MOD05_PRESUB, out_wv, xmap, ymap, ps, ns, nl
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Subset der Cloud-Daten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
MOD05_PRESUB, out_cld, xmap, ymap, ps, ns, nl
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der Wolkenmaske
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
MOD05_CALC_CLD, out_cld
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Umrechung der WV-Daten in cm/Luftsule Wasserinhalt
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
MOD05_CALC_WV, out_wv
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine MOD05_IMPORT_SINGLE_TS.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
; #########################################################
; MOD05_IMPORT_TS.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PRO MOD05_IMPORT_TS, pfad, out_wv, out_cld, ps, year
; Diese Routine dient der Ermittlung der einzelen
; Dateinamen in einem Zeitschritt (TS).
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Definition des Sucharguments fr Erfassung der
; hdf-Dateien
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
such_str = pfad + '\*.hdf'
names=FINDFILE(such_str, count=n_files)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung des Dateinames fr die Ausgabe
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
out_n=STRMID(names,STRPOS(names(0), '\',/REVERSE_SEARCH)+1)
out_name=STRMID(out_n,0,22)
print, 'Ausgabename',out_name(0)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Generelle Geoinformation fr importierte Daten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Rumliche Eingrenzung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
north_border=-15.0
east_border=18.0
south_border=-21.0
west_border=11.0
DEG_Borders = [west_border, east_border, $
			   north_border, south_border]
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
utm_zone = 33
datum = 'WGS84'
units = ENVI_TRANSLATE_PROJECTION_UNITS('Meters')
; ---------------------------------------------------------
; Anpassung der Eckkoordinaten
; ---------------------------------------------------------
ixmap=[DEG_Borders(0),DEG_Borders(1)]
iymap=[DEG_Borders(2),DEG_Borders(3)]
iproj = ENVI_PROJ_CREATE(/geographic)
; Get the projection information
oproj = ENVI_PROJ_CREATE(/utm, zone=utm_zone, /south, $
  datum=datum, units=units)
; Convert from Geographic to UTM
ENVI_CONVERT_PROJECTION_COORDINATES, $
  ixmap, iymap, iproj, $
  xkoord, ykoord, oproj
; ---------------------------------------------------------
; Bestimmung der ns und nl
; ---------------------------------------------------------
ns=FLOOR((xkoord(1) - xkoord(0))/ ps(0))
nl=FLOOR((ykoord(0) - ykoord(1))/ ps(1))
xmap=xkoord(0)
ymap=ykoord(0)
print, 'ns:', ns, 'ns:', nl
; Beginn der Schleife zum Import der einzelnen Dateien
FOR i=0,n_files-1 DO BEGIN
	; Ausgabename fr WV
	wv_pfd = out_wv +'\'+ out_name(i) + '.wv'
	; Ausgabename fr CLD
	cld_pfd = out_cld +'\'+ out_name(i) + '.cld'
	;Result = EOS_QUERY( name, Info)
	print, cld_pfd
	; Beginn des einzelnen Imports
	MOD05_IMPORT_SINGLE_TS, names(i), wv_pfd, cld_pfd,$
							xmap, ymap,ps, ns, nl
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Beginn des Zusammenfassens zu 10-Tages-Kompositen
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Zusammenfassen der WV-Daten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
MOD05_COMBIME_TS_WV, out_wv, year
MOD05_COMBIME_TS_CLD, out_cld, year
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine MOD05_IMPORT_TS.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
;##########################################################
; MOD05_COMBINE_TS_WV.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PRO MOD05_COMBIME_TS_WV, pfad, year
; Diese Routine dient der Zusammenfassun der einzelen
; WV-Dateien zu einem 10-Tages WV-Prokudt pro Zeitschritt
; (TS). Dabei werden die Water Vapor-Daten als einfaches
; atithmetisches Mittel ber die zehn Einzelwerte pro
; Pixel berechnet.
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Erzeugung des Dateinamens
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
print, pfad, year
TS = STRMID(pfad, STRPOS(pfad,'\',/REVERSE_SEARCH)+1,2)
yr = STRTRIM(year,2)
data_name = 'TSP_WV_' + TS + yr + '.data'
opfad = STRMID(pfad,0, STRPOS(pfad,'\',/REVERSE_SEARCH)+1)
out_name = opfad + data_name
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Erfassung der einzelnen Dateien
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
such_str = pfad + '\*.wv'
names=FINDFILE(such_str, count=n_files)
IF (n_files LE 0) THEN RETURN
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der ersten Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, names(0), r_fid=fid, /no_realize
  IF (fid EQ -1) THEN RETURN
ENVI_FILE_QUERY, fid, ns=ns, nl=nl, nb=nb, data_type=dt, $
				 bnames=bnames
pos=INTARR(nb)
dims = [-1, 0, ns-1, 0, nl-1]
map_info = ENVI_GET_MAP_INFO(fid=fid)
ENVI_FILE_MNG, id=fid, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der Daten zu Berechung und Abfangen von
; Pixelverschiebungen
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
data = FLTARR(ns,nl,n_files)
FOR i=0,n_files-1 DO BEGIN
  ENVI_OPEN_FILE, names(i), r_fid=fid, /no_realize
  data(*,*,i) = ENVI_GET_DATA(fid=fid, dims=dims, pos=0)
  ENVI_FILE_MNG, id=fid, /remove
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechung des Arithmetischen Mittels
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
data_out = FLTARR(ns,nl)
FOR y=0,nl-1 DO BEGIN
FOR x=0,ns-1 DO BEGIN
   dum = WHERE(data(x,y,*) NE 0, count)
  IF count NE 0 THEN BEGIN
   hilf=FLTARR(count)
   hilf(*)=data(x,y,dum)
   data_out(x,y) = MEAN(hilf)
  ENDIF ELSE BEGIN
   data_out(x,y)=0
  ENDELSE
ENDFOR
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe der 10-Tages_WaterVapor-Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
OPENW, 1, out_name
WRITEU,1, data_out
CLOSE,1
ENVI_SETUP_HEAD, fname=out_name, $
  ns=ns, nl=nl, nb=1, $
  interleave=0, data_type=dt, $
  offset=0, file_type=0, bnames=bnames, $
  map_info=map_info, /write, r_fid=oid
ENVI_FILE_MNG, id=oid, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine MOD05_COMBINE_TS_WV.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
;##########################################################
; MOD05_COMBINE_TS_CLD.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PRO MOD05_COMBIME_TS_CLD, pfad, year
; Diese Routine dient der Zusammenfassun der einzelen
; Cld-Dateien zu einem 10-Tages Wolkenprokudt pro
; Zeitschritt (TS). Dabei wird die Bewlkung ber 10 Tage
; als Prozentwert angegeben, damit der Bewlkungsverlauf
; besser widergespiegelt wird.
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Erzeugung des Dateinamens
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;pfad = 'e:\test\2001\wv\01'
TS = STRMID(pfad, STRPOS(pfad,'\',/REVERSE_SEARCH)+1,2)
yr = STRTRIM(year,2)
data_name = 'TSP_CLD_' + TS + yr + '.data'
;data_name = 'TSP_CLD_' + '08' + '2001' + '.data'
opfad = STRMID(pfad,0, STRPOS(pfad,'\',/REVERSE_SEARCH)+1)
out_name = opfad + data_name
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Erfassung der einzelnen Dateien
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
such_str = pfad + '\*.cld'
names=FINDFILE(such_str, count=n_files)
IF (n_files LE 0) THEN RETURN
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der ersten Datei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ENVI_OPEN_FILE, names(0), r_fid=fid, /no_realize
  IF (fid EQ -1) THEN RETURN
ENVI_FILE_QUERY, fid, ns=ns, nl=nl, nb=nb, data_type=dt, $
				 bnames=bnames
pos=INTARR(nb)
dims = [-1, 0, ns-1, 0, nl-1]
map_info = ENVI_GET_MAP_INFO(fid=fid)
ENVI_FILE_MNG, id=fid, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der Daten zu Berechung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
data = BYTARR(ns,nl,n_files)
FOR i=0,n_files-1 DO BEGIN
  ENVI_OPEN_FILE, names(i), r_fid=fid, /no_realize
  data(*,*,i) = ENVI_GET_DATA(fid=fid, dims=dims, pos=0)
  ENVI_FILE_MNG, id=fid, /remove
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechung der Bewlkung ber 10 Tage
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
data_out = BYTARR(ns,nl)
dum = BYTARR(n_files)
FOR y=0,nl-1 DO BEGIN
FOR x=0,ns-1 DO BEGIN
 hilf = WHERE(data(x,y,*) EQ 1, count)
 data_out(x,y) = BYTE((FLOAT(count)/FLOAT(n_files))*100)
ENDFOR
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ausgabe der 10-Tages-Bewlkungsdatei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
OPENW, 1, out_name
WRITEU,1, data_out
CLOSE,1
ENVI_SETUP_HEAD, fname=out_name, $
  ns=ns, nl=nl, nb=1, $
  interleave=0, data_type=1, $
  offset=0, file_type=0, bnames=bnames, $
  map_info=map_info, /write, r_fid=oid
ENVI_FILE_MNG, id=oid, /remove
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine MOD05_COMBINE_TS_CLD.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END
;##########################################################
; M A S S E N _ P R O C E S S I N G
;##########################################################
PRO MOD05_PREP
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Pfad zur Angabe Verweisdatei auf Dateipfade
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
pfad='M:\modis_namibia\Jahrgang_2000\MOD05_L2_2000.txt'

ps = [1000.,1000.]
year = 2001
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Graphische Benutzeroberflche zur
; Auswahl der Importparameter
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
base=WIDGET_AUTO_BASE(title='RBM - MOD05_L2 Import')
  w1=WIDGET_BASE(base, /row, /frame)
	e1=WIDGET_OUTF(w1, uvalue='home', $
	   prompt='Please enter path to MOD05_L2 data:',$
	   default=pfad, /auto)
  w2=WIDGET_BASE(base, /row, /frame)
	p1 = WIDGET_PARAM(w2, dt=4, field=4, xsize=4, $
         default=ps(0), uvalue='x_pix', $
         prompt='Resampling X Pixel size [m]', /auto)
   	p2 = WIDGET_PARAM(w2, dt=4, field=4, xsize=4, $
         default=ps(1), uvalue='y_pix', $
         prompt='Resampling Y Pixel size [m]', /auto)
    p3 = WIDGET_PARAM(w2, dt=2, field=8, xsize=4, $
         default=year, uvalue='year', $
         prompt='Enter importing Year', /auto)
  w3=WIDGET_BASE(base, /row, /frame)
	r1=WIDGET_OUTF(w3, uvalue='out', /directory, $
	   prompt='Please enter output directory:',/auto)
result=AUTO_WID_MNG(base)
if (result.accept eq 0) then RETALL  ;exit by cancel-button
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Auslesen der GUI Informationen
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
pfad=(result.home)
;--------------------------
ps(0)=(result.x_pix)
ps(1)=(result.y_pix)
year=(result.year)
;--------------------------
outf=(result.out)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Bestimmung der Anzahl der Zeilen der Verweisdatei
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
OPENR,1,pfad
; Define a string variable:
A = ''
n = 0
; Loop until EOF is found:
WHILE NOT EOF(1) DO BEGIN
  READF, 1, A
  n=n+1
ENDWHILE
CLOSE, 1
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Definition eines Array zur Aufnahme der Dateipfade
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
pfade=STRARR(n)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einlesen der Dateipfade
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
OPENR,1,pfad
FOR i=0, n-1 DO BEGIN
  READF, 1, A
  pfade(i) = A
ENDFOR
CLOSE, 1
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Bestimmung eines Ausgabepfades
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (outf EQ '') THEN RETALL
outf_wv = outf + '\wv\'
outf_cld = outf + '\cld\'
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Beginn der Prozessierung
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
FOR i=0,n-1 DO BEGIN
	; +++++++++++++++++++++++++++++++++++++++++++++++++++++
	; Erzeugung der Ausgabeordner fr WV
	; +++++++++++++++++++++++++++++++++++++++++++++++++++++
	pfad = STRMID(pfade(i),(STRPOS(pfade(i), '\', /REVERSE_SEARCH))+1)
	outpf_wv = outf_wv + pfad
	; +++++++++++++++++++++++++++++++++++++++++++++++++++++
	; berprfung des Ausgabeordners fr WV
	; +++++++++++++++++++++++++++++++++++++++++++++++++++++
	dirtest=FILE_TEST(outpf_wv,/DIRECTORY)
		IF dirtest EQ 0 THEN FILE_MKDIR, outpf_wv
	out_wv=outpf_wv
	; +++++++++++++++++++++++++++++++++++++++++++++++++++++
	; Erzeugung der Ausgabeordner fr CLD
	; +++++++++++++++++++++++++++++++++++++++++++++++++++++
	outpf_cld = outf_cld + pfad
	; +++++++++++++++++++++++++++++++++++++++++++++++++++++
	; berprfung des Ausgabeordners fr CLD
	; +++++++++++++++++++++++++++++++++++++++++++++++++++++
	dirtest=FILE_TEST(outpf_cld,/DIRECTORY)
		IF dirtest EQ 0 THEN FILE_MKDIR, outpf_cld
	out_cld=outpf_cld
	; Starten der Prozessierung
	MOD05_IMPORT_TS, pfade(i), out_wv, out_cld, ps, year
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ende der Routine MOD05_PREP
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END