; MOD09_PREP.pro
; ---------------------------------------------------------
; Das Tool MOD09_PREP dienst der Aufbereitung der MODIS
; MOD09 V004 Daten zur Verwendung im Regionalen Biosphren-
; modell (RBM) Kaokoveld.
; Dabei arbeitet die Routine mit verschieden Unterroutinen,
; die folgende Aktionen durchfhren:
; 1. Umprojektion der einzelnen MOD09-Daten aus ISIN nach
;    UTM
; 2. Ausgabe der unprojezierten Daten in ein temporres
;    Verzeichnis.
; 3. Zusammenfassen der Daten zu 10-Tages-Kompositen. Dabei
;    werden Wolken und eventuelle No-Data Bereiche ber den
;    zeitlichen Verschnitt eliminiert.
; *********************************************************
PRO PARAM, datname_in, tile_name, temp, pix
; ---------------------------------------------------------
; diese Programm erzeugt eine Parmeterfile um die C-Routine
; resample.exe zu steuern
; Erstellt am 28.02.2002 von J. Richters, RSRG - Uni Bonn
; ---------------------------------------------------------
; Folgende Variablen werden von einer bergeordneten
; Routine bergeben:
; datname_in: Dies ist der Dateiname samt Pfad der MODIS
; *.hdf Datei, die umprojetiert werden soll.
; ---------------------------------------------------------
parampfad=temp+'\project.prm'
datain='INPUT_FILENAME = '+datname_in
dataout='OUTPUT_FILENAME = '+temp+'modis_temp.tif'
P0='SPECTRAL_SUBSET = (1 1 1 1 1 1 1 0 0 0)' 	;folgende mit 1 markierte Kanle werden prozessiert
P1=' '
P2='SPATIAL_SUBSET_TYPE = INPUT_LINE_SAMPLE' 	;Definition eines Subsets als Zeilen/Spalten Angabe
case tile_name of
'h19v10': subset=[200,1300,2000,2400]
'h19v11': subset=[200,1,2000,1000]
endcase
uly=STRTRIM((subset(0)),1)
ulx=STRTRIM((subset(1)),1)
lry=STRTRIM((subset(2)),1)
lrx=STRTRIM((subset(3)),1)
P3='SPATIAL_SUBSET_UL_CORNER = ( '+ulx+' '+uly+' )'
P4='SPATIAL_SUBSET_LR_CORNER = ( '+lrx+' '+lry+' )'
P5=' '
P6='RESAMPLING_TYPE = NN'
P7=' '
P8='OUTPUT_PROJECTION_TYPE = UTM'
P9=' '
P10='UTM_ZONE = 33'
P11='DATUM =  WGS84'
P12='OUTPUT_PIXEL_SIZE = '+STRTRIM(pix,1)
OPENW,1,parampfad
PRINTF,1,datain
PRINTF,1,P1
PRINTF,1,P1
PRINTF,1,P0
PRINTF,1,P1
PRINTF,1,P2
PRINTF,1,P3
PRINTF,1,P4
PRINTF,1,dataout
PRINTF,1,P5
PRINTF,1,P6
PRINTF,1,P7
PRINTF,1,P8
PRINTF,1,P9
PRINTF,1,P10
PRINTF,1,P11
PRINTF,1,P12
PRINTF,1,P1
CLOSE,1
; ---------------------------------------------------------
; Ende der Routine PARAM.pro
; ---------------------------------------------------------
END
; *********************************************************
; *********************************************************
PRO RESAMPLE, datname_in, tile_name, temp, pix
; *********************************************************
; Diese Programm stt den Resample-Prozess der MODIS-
; Daten an.
; ---------------------------------------------------------
PARAM, datname_in, tile_name, temp, pix
; ---------------------------------------------------------
; Erzeugung einer Batchdatei zur Initialisierung der
; Umprojektion
; ---------------------------------------------------------
pre_command='set MRTDATADIR=D:\ENVI_IDL\startup\mrt23\data'
command='D:\ENVI_IDL\startup\mrt23\bin\resample -p '+temp+'project.prm -g ' +temp+'modis_resample.log'
bat_name='D:\ENVI_IDL\startup\mrt23\bin\ModisRun.bat'
OPENW,1,bat_name
PRINTF,1,pre_command
PRINTF,1,command
CLOSE,1
; ---------------------------------------------------------
; Absetzten des Shell-Befehls
; ---------------------------------------------------------
SPAWN, bat_name
; ---------------------------------------------------------
; Ende der Routine RESAMPLE.pro
; ---------------------------------------------------------
END
; *********************************************************
PRO MOD_PROJECT, datname_in, outpfad, temp, pix
; *********************************************************
; diese Programm erzeugt projeziert mit Hilfe der C-Routine
; resample.exe MODIS Daten aus einer ISIN Integerized
; Sinusodial Grid in UTM, WGS84, Zone 33 South um.
; Erstellt am 28.02.2002 von J. Richters, RSRG - Uni Bonn
; ---------------------------------------------------------
;Pfad zu dem temprr erzeugten Daten
; ---------------------------------------------------------
temp_pfad=temp+'*.tif'
; ---------------------------------------------------------
; Definition des Ausgabenamens
; ---------------------------------------------------------
datname = STRMID(datname_in, (STRPOS( datname_in, 'MOD', /REVERSE_SEARCH)))
prod_name=STRMID(datname, 0,5)
datum_name=STRMID(datname, (STRPOS( datname, 'A',5 ))+1,7)
tile_name=STRMID(datname, (STRPOS( datname, 'h',10 )),6)
outname=outpfad + prod_name + '_' + datum_name + '_' + tile_name + '.modis'
PRINT, outname
; ---------------------------------------------------------
; Aufruf von resample.pro, die eine Steuerdatei fr die
; C-Routine erzeugt und dem Resamling Prozess anstt.
; ---------------------------------------------------------
RESAMPLE, datname_in, tile_name, temp, pix
; ---------------------------------------------------------
; Abfragen der Anzahl der erzeugten Dateien.
; ---------------------------------------------------------
fname=FINDFILE(temp_pfad, count=count)
IF count LT 0 THEN RETURN
; ---------------------------------------------------------
; Oeffnen der ersten Datei um die Geometrie anzufragen
; ---------------------------------------------------------
ENVI_OPEN_DATA_FILE, fname(0), /tiff, r_fid=fid
  IF (fid EQ -1) THEN RETURN
ENVI_FILE_QUERY, fid, ns=ns, nl=nl, nb=nb
dims = [-1, 0, ns-1, 0, nl-1]
map_info=envi_get_map_info(fid=fid)
ENVI_FILE_MNG, id=fid, /remove
; ---------------------------------------------------------
; Erzeugen eines Array, in dem die umprojezierten Daten
; gelagert werden.
; ---------------------------------------------------------
data=INTARR(ns,nl,count)
; ---------------------------------------------------------
; Einlesen einzelen Bnder und Ausgabe in Speicherarray
; ---------------------------------------------------------
FOR i=0,count-1 DO BEGIN
ENVI_OPEN_DATA_FILE, fname(i), r_fid=fid, /tiff
data(*,*,i)=FIX(ENVI_GET_DATA(fid=fid, dims=dims, pos=0))
ENVI_FILE_MNG, id=fid, /remove , /delete
endfor
; ---------------------------------------------------------
; Ausgabe der umprojezierten Datei
; ---------------------------------------------------------
OPENW,2,outname
WRITEU, 2, data
CLOSE, 2
ENVI_SETUP_HEAD, fname=outname, $
  ns=ns, nl=nl, nb=count, $
  interleave=0, data_type=2, $
  offset=0, map_info=map_info, $
  /write, r_fid=out_id
ENVI_FILE_MNG, id=out_id, /remove
; ---------------------------------------------------------
; Ende der Routine MOD_PROJECT.pro
; ---------------------------------------------------------
END
; *********************************************************
PRO MOD09_HDF_IMPORT_ALL, in_pfad, outpfad, temp, pix
; *********************************************************
; Das Programm arbeitet in folgender Reihenfolge:
; 1. 	Abfrage des Verzeichnisses, in dem die MODIS Rohdaten liegen (Input)
; 		und Angabe des Ausgabeordners fr prozessierten Daten.
; 2. 	bergabe der Dateinamen einzeln an die Subroutine mod_reproject.pro
;		Hier wird der Ausgabedateiname abgeleitet, die zugehrige Parameter-
;		Datei fr die Projektion geschrieben und die C-Routine resample.exe
; 		gestartet. Anschlieend werden die erzeugten *.tif-Dateien erneut
;		in ENVI eingelesen und als ENVI-Datei ausgegeben.
; 3.	Zusamenfgen der beiden zu einem Zeitpunkt gehrigen Datenfiles
; 		ber die Mosaiking-Funktion von ENVI.
; ---------------------------------------------------------
; Definition eines temporren Verzeichnisses
; ---------------------------------------------------------
tmp=temp+'\MODIS'		; Temporres Verzeichnis zur
							; Zwischenlagerung von Daten
; ---------------------------------------------------------
; berprfung der Temporren Ordner
; ---------------------------------------------------------
dirtest=FILE_TEST(tmp,/DIRECTORY)
	if dirtest eq 0 then FILE_MKDIR, tmp
tmp=tmp+'\'
; ---------------------------------------------------------
; Eingabename der MODIS +.hdf Datei
; ---------------------------------------------------------
such_str=in_pfad+'\*.hdf'
data_names=FINDFILE(such_str, count=filezahl)
IF filezahl LT 1 THEN GOTO, ende
; ---------------------------------------------------------
; Beginn der Prozessierung
; ---------------------------------------------------------
FOR i=0, filezahl-1 DO BEGIN
	MOD_PROJECT, data_names(i), outpfad, tmp, pix
ENDFOR
; ---------------------------------------------------------
; Ende der Prozessierung erreicht
; ---------------------------------------------------------
ende: print,'No File found'
; ---------------------------------------------------------
; Ende der Routine MOD09_HDF_IMPORT_ALL.pro
; ---------------------------------------------------------
END
; *********************************************************
PRO CLOUD, data, wolk
; *********************************************************
; Diese Routine testen des Vorhandensein von Wolken und
; erzeugt eine Wolkenmaske aus den Spektraldaten.
; ---------------------------------------------------------
; Definition eines Arrays um Zwischenergebnisse Test1
; aufzunehmen
; ---------------------------------------------------------
wolk = BYTARR(N_ELEMENTS(data(*,0)),N_ELEMENTS(data(0,*)))
; ---------------------------------------------------------
; Testabfrage
; ---------------------------------------------------------
g1=5500
	hilf=WHERE(data(*,*) gt g1, count)
	wolk(*,*)=0
	IF (count GT 0) THEN wolk(hilf)=1
; ---------------------------------------------------------
; Ende der Routine CLOUD.pro
; ---------------------------------------------------------
END
; *********************************************************
PRO NO_DATA, data, no_mask
; *********************************************************
; Diese Routine berprft, ob alle Pixel Werte enthalten
; und erzeugt eine No-Data-Maske.
; ---------------------------------------------------------
; Definition eines Arrays um Zwischenergebnisse Test1
; aufzunehmen
; ---------------------------------------------------------
no_mask = BYTARR(N_ELEMENTS(data(*,0)),N_ELEMENTS(data(0,*)))
; ---------------------------------------------------------
; Testabfrage
; ---------------------------------------------------------
ndv=(0)						; ndv = no data value
	hilf=WHERE(data(*,*) eq ndv, count)
	no_mask(*,*)=0
	IF (count GT 0) THEN no_mask(hilf)=1
; ---------------------------------------------------------
; Ende der Routine NO_DATA.pro
; ---------------------------------------------------------
END
; ********************************************************
PRO MERGE_MASK, wolk, nodata, mask
; *********************************************************
; Zusammenfuehren der Wolken & NO Data Maske
; ---------------------------------------------------------
mask=BYTARR(N_ELEMENTS(wolk(*,0)), N_ELEMENTS(wolk(0,*)))
hilf2=wolk(*,*)+nodata(*,*)
hilf=WHERE(hilf2(*,*) GT 0,count)
mask(*,*)=0
IF (count GT 0)THEN mask(hilf)=1
; ---------------------------------------------------------
; Ende der Routine MERGE_MASK.pro
; ---------------------------------------------------------
END
; ********************************************************
PRO MITTEL_WERT, data,mask, erg
; *********************************************************
; Berechung der Mittelwert pro MODIS-Band unter Einbezieh-
; ung der Gesamtmaske.
; ---------------------------------------------------------
erg=INTARR(N_ELEMENTS(data(*,0,0)),N_ELEMENTS(data(0,*,0)))
FOR x=0,(N_ELEMENTS(data(*,0,0)))-1 DO BEGIN
	FOR y=0,(N_ELEMENTS(data(0,*,0)))-1 DO BEGIN
		hilf=WHERE(mask(x,y,*) ne 1,count)
		IF count NE 0 THEN BEGIN
		werte=INTARR(10)
		werte=data(x,y,*)
		erg(x,y)=ROUND(MEDIAN(werte))
		ENDIF ELSE BEGIN
		erg(x,y)=-9999
		ENDELSE
	ENDFOR
ENDFOR
; ---------------------------------------------------------
; Ende der Routine MITTEL_WERT.pro
; ---------------------------------------------------------
END
; *********************************************************
PRO MOD09_COMPOSIT, in_pfad, outpfad
; ---------------------------------------------------------
; Die Routine MOD09_COMPOSIT.pro bildet aus mehreren
; importierten MOD09-Daten 10-Tages-Komposite. Dabei werden
; sowohl eventuell vorhandene Wolken genauso wie Fehlpixel
; reduziert bzw. beseitigt.
; ---------------------------------------------------------
such_str=in_pfad+'\*.modis'
data_names=FINDFILE(such_str, count=count)
IF count LT 1 THEN RETURN
; ---------------------------------------------------------
; Erzeugen des Ausgabenamens der Datei
; ---------------------------------------------------------
date=STRMID(data_names(0),(STRPOS(data_names(0),'\',/REVERSE_SEARCH)))
Prod=STRMID(date,(STRPOS(date,'M')),5)
year=STRMID(date,7,4)
period_hlp=STRMID(data_names(0),0,(STRPOS(data_names(0),'\',/REVERSE_SEARCH)))
period=STRMID(period_hlp,(STRPOS(period_hlp,'_',/REVERSE_SEARCH))+1,2)
oname='TSP_' + Prod + '_' + period + year + '.data'
outname=outpfad+oname
; ---------------------------------------------------------
; Einlesen von grundsaetzlichen Dateninformationen
; ---------------------------------------------------------
ENVI_OPEN_FILE, data_names(0), r_fid=fid, /NO_REALIZE
envi_file_query, fid, ns=ns, nl=nl, nb=nb
dims = [-1, 0, ns-1, 0, nl-1]
map_info=envi_get_map_info(fid=fid)
ENVI_FILE_MNG, id=fid, /remove
data=INTARR(ns,nl,nb,count)
; ----------------------------------------------------
; Erzeugen einer Gesamtvariablen,
; die alle Daten enthlt
; ----------------------------------------------------
FOR i=0,count-1 DO BEGIN
	ENVI_OPEN_FILE, data_names(i), r_fid=fid, /NO_REALIZE
	FOR j=0,nb-1 DO BEGIN
	 data(*,*,j,i)=ENVI_GET_DATA(fid=fid, dims=dims, pos=j)
	ENDFOR
	ENVI_FILE_MNG, id=fid, /remove
	PRINT, 'Datei Nr.',i,' eingelesen.'
ENDFOR
; ---------------------------------------------------------
; Ausmaskieren der Wolkenbereiche
; ---------------------------------------------------------
wolk=BYTARR(nl,ns,count)
FOR i=0,count-1 DO BEGIN
	file_data=data(*,*,1,i)
	CLOUD, file_data,wolk_mask
	wolk(*,*,i)=wolk_mask
	PRINT, 'Wolken fr Datei Nr.',i,'ermittelt.'
ENDFOR
; ---------------------------------------------------------
; Ausmaskieren der No-Data-Bereiche
; ---------------------------------------------------------
nodata=BYTARR(nl,ns,count)
FOR i=0,count-1 DO BEGIN
	file_data=data(*,*,1,i)
	NO_DATA, file_data,no_mask
	nodata(*,*,i)=no_mask
	PRINT, 'NO DATA Werte fr Datei Nr.',i,'ermittelt.'
ENDFOR
; ---------------------------------------------------------
; Bestimmung der Gesamtmaske (aus Wolken und No Data)
; ---------------------------------------------------------
mask=BYTARR(N_ELEMENTS(data(*,0,0,0)),N_ELEMENTS(data(0,*,0,0)),count)
FOR i=0,count-1 DO BEGIN
	wolk_mask=wolk(*,*,i)
	nodata_mask=nodata(*,*,i)
	MERGE_MASK, wolk_mask, nodata_mask, erg_mask
	mask(*,*,i)=erg_mask
	PRINT, 'Zusammenfhren der Masken fr Datei Nr.',i,'beendet.'
ENDFOR
; ---------------------------------------------------------
; Ermittlung der Durchschnittswerte
; ---------------------------------------------------------
outdata=INTARR(ns,nl,nb)
data_band=INTARR(ns,nl,count)
FOR b=0,nb-1 DO BEGIN
	FOR n=0,count-1 DO BEGIN
		data_hilf=data(*,*,b,n)
		data_band(*,*,n)=data_hilf(*,*)
	ENDFOR
	MITTEL_WERT, data_band,mask, erg
	outdata(*,*,b)=erg
	PRINT, 'Die Mittelwertsberechung fr Band Nr.',b,'ist abgeschlossen.'
ENDFOR
; ---------------------------------------------------------
; Ausgabe der gemittelten Datei
; ---------------------------------------------------------
OPENW,1,outname
WRITEU,1,outdata
CLOSE,1
ENVI_SETUP_HEAD, fname=outname, $
  ns=ns, nl=nl, nb=nb, $
  interleave=0, data_type=2, $
  offset=0, map_info=map_info, $
  /write, r_fid=o_id
ENVI_FILE_MNG, id=o_id, /remove
; ---------------------------------------------------------
; Ende der Routine MOD09_COMPOSIT.pro
; ---------------------------------------------------------
end
; *********************************************************
PRO MOD09_PREP
; *********************************************************
; Die Routine MOD09_PREP.pro liest beliebige MODIS MOD09
; Daten (Surface Reflectance) ein und bereitet diese
; allgemein fr die Anwendung im RBM Kaokoveld auf.
; Dabei bernimmt die Routine MOD09_PREP.pro eine steuernde
; Funktion und ruft die nachgeordneten Prozesse zur
; Datenverarbeitung auf.
; ---------------------------------------------------------
; VORDAFINIERTE VARIABLEN
; Diese sollen nur bei der Installation auf einer neuen
; Plattfrom angepasst werden !!
; ---------------------------------------------------------
tmp = 'D:\Temp'			; Pfad zu temporren Verzeichnis
						; zur Zwischenlagerung von Daten.
; ---------------------------------------------------------
; Eingangspfad
; ---------------------------------------------------------
pfad='E:\MOD09_2002_test\MOD09_test_2002.txt'
pix = 1000
; ---------------------------------------------------------
; Graphische Benutzeroberflche zur Eingabe der Pfade
; ---------------------------------------------------------
list = ['500','1000']
base=WIDGET_AUTO_BASE(title='RBM - MOD09 Data Preparation')
  w1=WIDGET_BASE(base, /row, /frame)
	e1=WIDGET_OUTF(w1, uvalue='ipfad', default=pfad, $
	   prompt='Please enter MOD09 directory file:', /auto)
    w3=WIDGET_BASE(base, /column, /frame)
   	p5 = WIDGET_OUTF(w3, uvalue='opfad', /directory, $
   	     prompt='Please enter output directory:', /auto)
	p6 = widget_toggle(w3, uvalue='pix', list=list, $
   	     prompt='Select import pixel resolution [m]', /auto)
result=AUTO_WID_MNG(base)
if (result.accept eq 0) then RETALL  ;exit by cancel-button
; ---------------------------------------------------------
; Auslesen der GUI
; ---------------------------------------------------------
pfad=(result.ipfad)
;--------------------------
outf=(result.opfad)
pixel=(result.pix)
CASE pixel OF
0 :pix=500
1:pix=1000
ENDCASE
; ---------------------------------------------------------
; Definition der Ausgabe
; ---------------------------------------------------------
outf=outf+'\'
mod09_outpfad = outf
dirtest=FILE_TEST(mod09_outpfad,/DIRECTORY)
 if dirtest EQ 0 THEN FILE_MKDIR, mod09_outpfad
mod09_outpfad=mod09_outpfad+'\'
; ---------------------------------------------------------
; Bestimmung der Lnge der Pfaddatei
; ---------------------------------------------------------
A = ''		; Dummy-Variable zum Einlesen der Pfade
n = 0		; Zhlvariable z. Ermitt. d.Gesamtzahl d. Pfade
; ---------------------------------------------------------
; Schleife mit EOF Statment um Lnge zu Bestimmen
; ---------------------------------------------------------
OPENR,1,pfad
WHILE NOT EOF(1) DO BEGIN
 READF, 1, A
 n=n+1
ENDWHILE
CLOSE, 1
; ---------------------------------------------------------
; Echtes Einlesen der Pfadangaben
; ---------------------------------------------------------
pfade=STRARR(n)
OPENR,1,pfad
FOR i=0, n-1 DO BEGIN
 READF, 1, A 				; Read a line of text
 pfade(i) = A
ENDFOR
CLOSE, 1
; ---------------------------------------------------------
; B E G I N N   D E R   P R O Z E S S I E R U N G
; ---------------------------------------------------------
; Erzeugung eines Ordner, der die Importprodukte aufnimmt
; ---------------------------------------------------------
pfad = STRMID(pfade,(STRPOS(pfade(0), '\', /REVERSE_SEARCH))+1)
import_outpfad=tmp +'\'+ pfad
; ---------------------------------------------------------
; Beginn der Importschleife
; ---------------------------------------------------------
FOR i=0,n-1 DO BEGIN
; ---------------------------------------------------------
; berprfung des Ausgabeordners
; ---------------------------------------------------------
 import_outpfd = import_outpfad(i)
 dirtest=FILE_TEST(import_outpfd,/DIRECTORY)
 if dirtest EQ 0 THEN FILE_MKDIR, import_outpfd
 import_outpfd = import_outpfd +'\'
; ---------------------------------------------------------
; Import der HDF-Daten
; ---------------------------------------------------------
  MOD09_HDF_IMPORT_ALL, pfade(i), import_outpfd, tmp, pix
; ---------------------------------------------------------
; Compositing
; ---------------------------------------------------------
  MOD09_COMPOSIT, import_outpfad(i), mod09_outpfad
; ---------------------------------------------------------
ENDFOR
; ---------------------------------------------------------
; Ende der Routine MOD09_PREP.pro
; ---------------------------------------------------------
PRINT, 'All Jobs finished !'
PRINT, 'Thanks for using MOD09_PREP'
PRINT, '(c) J.Richters, GIUB, Uni Bonn; 12/2003'
; ---------------------------------------------------------
; Ende der Routine MOD09_PREP
; ---------------------------------------------------------
END