; Tool zur automatischen Umprojektion von MODIS Daten mit
; Hilfe des MODIS Reprojection Tool V.2.2
; ---------------------------------------------------------
; Das Programm arbeitet in folgender Reihenfolge:
; 1. 	Abfrage des Verzeichnisses, in dem die MODIS
;		Rohdaten liegen (Input) und Angabe des Ausgabe-
;		ordners fr prozessierten Daten.
; 2. 	bergabe der Dateinamen einzeln an die Subroutine
;		mod_reproject.pro. Hier wird der Ausgabedateiname
;		abgeleitet, die zugehrige Parameter-Datei fr die
;		Projektion geschrieben und die C-Routine
;		resample.exe gestartet. Anschlieend werden die
;		erzeugten *.tif-Dateien erneut in ENVI eingelesen
;		und als ENVI-Datei ausgegeben.
; 3.	Zusamenfgen der beiden zu einem Zeitpunkt
;		gehrigen Datenfiles ber die Mosaiking-Funktion
;		von ENVI.
;**********************************************************
PRO PARAM, datname_in, tile_name, temp, pix
;**********************************************************
; diese Programm erzeugt eine Parmeterfile um die C-Routine
; resample.exe zu steuern
; Erstellt am 28.02.2002 von J. Richters, RSRG - Uni Bonn
; ---------------------------------------------------------
; Folgende Variablen werden von einer bergeordneten Routine
; bergeben: datname_in: Dies ist der Dateiname samt Pfad
; der MODIS *.hdf Datei, die umprojetiert werden soll.
; (STRING Variable!)
; ---------------------------------------------------------
; Definition der einzelen STRING-Eintrge fr *.prm-Steuer-
; datei fr resample.exe
; ---------------------------------------------------------
parampfad=temp+'project.prm'
datain='INPUT_FILENAME = '+datname_in
dataout='OUTPUT_FILENAME = '+temp+'modis_temp.tif'
P0='SPECTRAL_SUBSET = (1 0 0 0 1 0 0 0 1 1 0 0)' 	;folgende mit 1 markierte Kanle werden prozessiert
P1=' '
P2='SPATIAL_SUBSET_TYPE = INPUT_LINE_SAMPLE' 	;Definition eines Subsets als Zeilen/Spalten Angabe
CASE tile_name OF
'h19v10': subset=[1,600,1200,1200]
'h19v11': subset=[1,1,1200,600]
ENDCASE
uly=STRTRIM((subset(0)),1)
ulx=STRTRIM((subset(1)),1)
lry=STRTRIM((subset(2)),1)
lrx=STRTRIM((subset(3)),1)
P3='SPATIAL_SUBSET_UL_CORNER = ( '+ulx+' '+uly+' )'
P4='SPATIAL_SUBSET_LR_CORNER = ( '+lrx+' '+lry+' )'
P5=' '
P6='RESAMPLING_TYPE = NN'
P7=' '
P8='OUTPUT_PROJECTION_TYPE = UTM'
P9=' '
P10='UTM_ZONE = 33'
P11='DATUM =  WGS84'
P12='OUTPUT_PIXEL_SIZE = '+ STRTRIM(pix,1)
; ---------------------------------------------------------
; Ausgabe der Parameterdatei zur bergabe an resample.exe
; ---------------------------------------------------------
OPENW,1,parampfad
PRINTF,1,datain
PRINTF,1,P1
PRINTF,1,P1
PRINTF,1,P0
PRINTF,1,P1
PRINTF,1,P2
PRINTF,1,P3
PRINTF,1,P4
PRINTF,1,dataout
PRINTF,1,P5
PRINTF,1,P6
PRINTF,1,P7
PRINTF,1,P8
PRINTF,1,P9
PRINTF,1,P10
PRINTF,1,P11
PRINTF,1,P12
PRINTF,1,P1
CLOSE,1
; ---------------------------------------------------------
; Ende der Routine PARAM.pro
; ---------------------------------------------------------
END
;**********************************************************
PRO RESAMPLE, datname_in, tile_name, temp, pix
;**********************************************************
; Diese Programm stt den Resample-Prozess der MODIS
; Daten an und arbeitet dazu in zwei Schritten;
; 1. Die Routine PARAM.pro wird angestoen, die die
;    Parmeterdatei fr die Shell-Routine resample.exe erzeugt.
; 2. Die Shell-Routine resample.exe wird von der Routine
;    RESAMPLE.pro angestossen.
; ---------------------------------------------------------
; Erzeugen der Parameterdatei fr resample.exe
; ---------------------------------------------------------
PARAM, datname_in, tile_name, temp, pix
; ---------------------------------------------------------
; Erzeugung einer Batchdatei zur Initialisierung
; der Umprojektion. Dabei muss die Shellvariable
; MRTDATADIR gesetzt werden.
; ---------------------------------------------------------
pre_command='set MRTDATADIR=D:\ENVI_IDL\startup\mrt31\data'
command='D:\ENVI_IDL\startup\mrt31\bin\resample -p '+temp+'project.prm -g ' +temp+'modis_resample.log'
; ---------------------------------------------------------
; Definition des Dateinamens und Ausgabe der Batchdatei
; ---------------------------------------------------------
bat_name='D:\ENVI_IDL\startup\mrt31\bin\ModisRun.bat'
OPENW,1,bat_name
PRINTF,1,pre_command
PRINTF,1,command
CLOSE,1
; ---------------------------------------------------------
; Absetzten des Shell-Befehls und bergabe des
; Befehls an die Kommandozeile
; ---------------------------------------------------------
SPAWN, bat_name
; ---------------------------------------------------------
; Ende des Routine RESAMPLE.exe
; ---------------------------------------------------------
END
;**********************************************************
PRO MOD_PROJECT, datname_in, outpfad, temp, pix
;**********************************************************
; diese Programm erzeugt projeziert mit Hilfe der C-Routine
; resample.exe MODIS Daten aus einer ISIN Integerized
; Sinusodial Grid in UTM, WGS84, Zone 33 South um.
; Erstellt am 28.02.2002 von J. Richters, RSRG - Uni Bonn
; ---------------------------------------------------------
;Pfad zu dem temprr erzeugten Daten
; ---------------------------------------------------------
temp_pfad=temp+'*.tif'
; ---------------------------------------------------------
; Definition des Ausgabenamens
; ---------------------------------------------------------
	datname = STRMID(datname_in, (STRPOS( datname_in, 'MOD', /REVERSE_SEARCH)))
	prod_name=STRMID(datname, 0,5)
	datum_name=STRMID(datname, (STRPOS( datname, 'A2',5 ))+1,7)
	tile_name=STRMID(datname, (STRPOS( datname, 'h',10 )),6)
	;file_mkdir, outpfad+'reproj'
	LST_name=outpfad + prod_name + '_' + datum_name + '_' + tile_name + '_LST.modis'
	EMS_name=outpfad + prod_name + '_' + datum_name + '_' + tile_name + '_Emissivity.modis'
	print, LST_name
	print, EMS_name
; ---------------------------------------------------------
; Aufruf von resample.pro, die eine Steuerdatei fr die
; C-Routine erzeugt und dem Resamling Prozess anstt.
; ---------------------------------------------------------
RESAMPLE, datname_in, tile_name, temp, pix
; ---------------------------------------------------------
; Import der LST - Daten
; ---------------------------------------------------------
; Abfragen der Anzahl der erzeugten Dateien.
; ---------------------------------------------------------
LST_pfad=temp+'*1km.tif'
fname=FINDFILE(LST_pfad, count=count)
if count LT 0 THEN RETURN
; ---------------------------------------------------------
; Oeffnen der ersten Datei um die Geometrie anzufragen
; ---------------------------------------------------------
ENVI_OPEN_DATA_FILE, fname(0), /tiff, r_fid=fid
  if (fid eq -1) then RETURN
envi_file_query, fid, ns=ns, nl=nl, nb=nb, data_type=dt
dims = [-1, 0, ns-1, 0, nl-1]
map_info=envi_get_map_info(fid=fid)
envi_file_mng, id=fid, /remove
; ---------------------------------------------------------
; Erzeugen eines Array, in dem die umprojezierten Daten
; gelagert werden.
; ---------------------------------------------------------
data=INTARR(ns,nl,count)
; ---------------------------------------------------------
; Einlesen einzelen Bnder und ausgabe in Speicherarray
; ---------------------------------------------------------
for i=0,count-1 do begin
envi_open_data_file, fname(i), r_fid=fid, /tiff
 if (fid eq -1) then RETURN
data(*,*,i)=(ENVI_GET_DATA(fid=fid, dims=dims, pos=0))
envi_file_mng, id=fid, /remove , /delete
endfor
; ---------------------------------------------------------
; Ausgabe der LST Daten
; ---------------------------------------------------------
openw,2,LST_name
writeu, 2, data
close, 2
ENVI_SETUP_HEAD, fname=LST_name, $
  ns=ns, nl=nl, nb=count, bnames=['LST Day [C]','LST Night [C]'], $
  interleave=0, data_type=dt, $
  offset=0, map_info=map_info, $
  /write, r_fid=LST_id
envi_file_mng, id=LST_id, /remove
; ---------------------------------------------------------
; Import der EMS - Daten
; ---------------------------------------------------------
; Abfragen der Anzahl der erzeugten Dateien.
; ---------------------------------------------------------
LST_pfad=temp+'*Emis_3*.tif'
fname=findfile(LST_pfad, count=count)
if count lt 0 then return
; ---------------------------------------------------------
; Oeffnen der ersten Datei um die Geometrie anzufragen
; ---------------------------------------------------------
envi_open_data_file, fname(0), /tiff, r_fid=fid
  if (fid eq -1) then return
envi_file_query, fid, ns=ns, nl=nl, nb=nb, data_type=dt
dims = [-1, 0, ns-1, 0, nl-1]
map_info=envi_get_map_info(fid=fid)
envi_file_mng, id=fid, /remove
; ---------------------------------------------------------
; Erzeugen eines Array, in dem die importierten Daten
; gelagert werden.
; ---------------------------------------------------------
data=bytarr(ns,nl,count)
; ---------------------------------------------------------
; Einlesen einzelen Bnder und ausgabe in Speicherarray
; ---------------------------------------------------------
for i=0,count-1 do begin
envi_open_data_file, fname(i), r_fid=fid, /tiff
 if (fid eq -1) then return
data(*,*,i)=(ENVI_GET_DATA(fid=fid, dims=dims, pos=0))
envi_file_mng, id=fid, /remove , /delete
endfor
; ---------------------------------------------------------
; Ausgabe der LST Daten
; ---------------------------------------------------------
openw,2,EMS_name
writeu, 2, data
close, 2
ENVI_SETUP_HEAD, fname=EMS_name, $
  ns=ns, nl=nl, nb=2, bnames=['Emis'], $
  interleave=0, data_type=1, $
  offset=0, map_info=map_info, $
  /write, r_fid=LST_id
envi_file_mng, id=LST_id, /remove
; ---------------------------------------------------------
; Ende der Routine MOD_PROJECT.pro
; ---------------------------------------------------------
END

;**********************************************************
PRO MOD11_HDF_IMPORT, in_pfad, outpfad, pix, temp
;**********************************************************
; Diese Routine steuert die Auswahl des Verzeichnisses,
; das an die untergeordnete Routine mod_project bergeben
; wird.
; ---------------------------------------------------------
; berprfung der Existenz der temporren Ordner
; ---------------------------------------------------------
dirtest=FILE_TEST(temp,/DIRECTORY)
	IF dirtest eq 0 THEN FILE_MKDIR, temp
temp=temp+'\'
tmp=temp+'mod_resample'
dirtest=FILE_TEST(tmp,/DIRECTORY)
	IF dirtest eq 0 THEN FILE_MKDIR, tmp
tmp=tmp+'\
; ---------------------------------------------------------
; Eingabename der MODIS +.hdf Datei
; ---------------------------------------------------------
such_str=in_pfad+'\*.hdf'
data_names=FINDFILE(such_str, count=filezahl)
IF filezahl LT 1 THEN GOTO, ende
; ---------------------------------------------------------
; Beginn der Prozessierung
; ---------------------------------------------------------
FOR i=0, filezahl-1 DO BEGIN
	MOD_PROJECT, data_names(i), outpfad, temp, pix
ENDFOR
; ---------------------------------------------------------
; Ende der Prozessierung erreicht
; ---------------------------------------------------------
ende: PRINT,'No File found'
; ---------------------------------------------------------
; Ende der Routine MOD11_HDF_IMPORT.pro
; ---------------------------------------------------------
END
; *********************************************************
; *** BEGINN DER ROUTINE MOD11_COMPOSIT *******************
; *********************************************************
; *** NO DATA MASKIERUNG **********************************
; *********************************************************
PRO NO_DATA, data, no_mask
; ---------------------------------------------------------
; Definition eines Arrays um Zwischenergebnisse Test1
; aufzunehmen
; ---------------------------------------------------------
no_mask = BYTARR(N_ELEMENTS(data(*,0)),N_ELEMENTS(data(0,*)))
; ---------------------------------------------------------
; Testabfrage
; ---------------------------------------------------------
ndvl=7500						; ndvl = no data value low
ndvh=65535						; ndvh = no data value high
no_mask(*,*)=0
	hilf=WHERE(data(*,*) LT ndvl , count)
	IF (count NE 0) THEN no_mask(hilf)=1
	hilf2=WHERE(data(*,*) GT ndvh,count2)
	IF (count2 NE 0) THEN no_mask(hilf2)=1
; ---------------------------------------------------------
; Ende der Routine NO_DATA.pro
; ---------------------------------------------------------
END
; *********************************************************
; *** NO DATA MASKIERUNG EMS ******************************
; *********************************************************
PRO NO_DATA_EMS, data, no_mask
; *********************************************************
; Definition eines Arrays um Zwischenergebnisse Test1 aufzunehmen
no_mask = bytarr(n_elements(data(*,0)),n_elements(data(0,*)))
; Testabfrage
ndvl=0						; ndvl = no data value low
no_mask(*,*)=0
	hilf=where(data(*,*) eq ndvl , count)
	if (count NE 0) then no_mask(hilf)=1
;no_mask(*,*)=0
END
; *********************************************************
; *** MITTELWERT BERECHNUNG *******************************
; *********************************************************
PRO MITTEL, t_dat, t_mask, erg
; *********************************************************
hilf=WHERE(t_mask eq 0, count)
if count NE 0 then $
erg=FIX(MEAN(t_dat(hilf))) else erg=0
END
; *********************************************************
; *** MAIN PROCESSING LEVEL *******************************
; *********************************************************
PRO MOD11_TEMP_COMPOSIT, in_pfad, temp_outpfad
; ---------------------------------------------------------
; Eingabename der MODIS +.modis Datei
; ---------------------------------------------------------
; Massenprozessierung
; ---------------------------------------------------------
such_str=in_pfad+'\*LST.modis'
data_name=findfile(such_str, count=count)
tile_name=STRMID(data_name, (STRPOS( data_name(0), 'h',10 )),6)
use=where(tile_name eq 'h19v10', count_use)
if count_use lt 1 then return
data_names=strarr(count_use)
data_names=data_name(use)
count=count_use
; ---------------------------------------------------------
; Erzeugen des Ausgabenamens der Datei
; ---------------------------------------------------------
date=STRMID(data_names(0),(STRPOS(data_names(0),'\',/REVERSE_SEARCH)))
Prod=STRMID(date,(STRPOS(date,'M')),5)
year=STRMID(date,7,4)
period_hlp=STRMID(data_names(0),0,(STRPOS(data_names(0),'\',/REVERSE_SEARCH)))
period=STRMID(period_hlp,(STRPOS(period_hlp,'_',/REVERSE_SEARCH))+1,2)
Temp_oname='TSP_TEMP' + '_' + period + year + '.data'
temp_name=temp_outpfad+Temp_oname
print, temp_name
; ---------------------------------------------------------
; Einlesen von grundsaetzlichen Dateninformationen
; ---------------------------------------------------------
envi_open_file, data_names(0), r_fid=fid, /NO_REALIZE
envi_file_query, fid, ns=ns, nl=nl, nb=nb
dims = [-1, 0, ns-1, 0, nl-1]
map_info=envi_get_map_info(fid=fid)
envi_file_mng, id=fid, /remove
; ---------------------------------------------------------
; Erfassung der Temperatur
; ---------------------------------------------------------
temp_data=intarr(ns,nl,count)
temp_mask=bytarr(ns,nl,count)
day_temp=intarr(ns,nl)
night_temp=intarr(ns,nl)
for i=0,count-1 do begin
; ---------------------------------------------------------
; Einlesen der einzelnen Datei
; ---------------------------------------------------------
  envi_open_file, data_names(i), r_fid=fid, /NO_REALIZE
	day_temp(*,*)=ENVI_GET_DATA(fid=fid, dims=dims, pos=0)
	night_temp(*,*)=ENVI_GET_DATA(fid=fid, dims=dims, pos=1)
	; mit pos=0 wird das 1. Band selectiert
  envi_file_mng, id=fid, /remove ;,/delete
  print, 'Temp_Datei: ',i,' eingelesen.'
; ---------------------------------------------------------
; Ausmaskieren der No-Data-Bereiche
; ---------------------------------------------------------
  day_nodata=bytarr(ns,nl)
  no_data, day_temp,no_mask
  day_nodata(*,*)=no_mask
; ---------------------------------------------------------
  night_nodata=bytarr(ns,nl)
  no_data, night_temp,no_mask
  night_nodata(*,*)=no_mask
; ---------------------------------------------------------
; Berechung der mittleren Tagestemperatur
; ---------------------------------------------------------
  temp_dum=intarr(ns,nl)
  temp_dum(*,*)=0
  mask_dum=bytarr(ns,nl)
  mask_dum(*,*)=0
; ---------------------------------------------------------
  hilf=WHERE((day_nodata(*,*) eq 0) and $
			(night_nodata(*,*) eq 0),c_hilf)
  if c_hilf NE 0 then begin
	temp_dum(hilf)=ROUND((day_temp(hilf)+night_temp(hilf))/2)
	mask_dum(hilf)=0
  endif
; ---------------------------------------------------------
  hilf=WHERE((day_nodata(*,*) eq 0) and $
			(night_nodata(*,*) eq 1),c_hilf)
  if c_hilf NE 0 then begin
	temp_dum(hilf)=ROUND(day_temp(hilf))
	mask_dum(hilf)=0
  endif
; ---------------------------------------------------------
  hilf=WHERE((day_nodata(*,*) eq 1) and $
			(night_nodata(*,*) eq 0),c_hilf)
  if c_hilf NE 0 then begin
	temp_dum(hilf)=0
	mask_dum(hilf)=1
  endif
; ---------------------------------------------------------
  hilf=WHERE((day_nodata(*,*) eq 1) and $
			(night_nodata(*,*) eq 1),c_hilf)
  if c_hilf NE 0 then begin
	temp_dum(hilf)=0
	mask_dum(hilf)=1
  endif
; ---------------------------------------------------------
  temp_data(*,*,i)=temp_dum
  temp_mask(*,*,i)=mask_dum
endfor
temp_dum=0
; ---------------------------------------------------------
; Bestimmung der mittleren Temperatur ber 10 Tage
; ---------------------------------------------------------
temp_out=intarr(ns,nl)
t_dat=intarr(count)
t_mask=bytarr(count)
for y=0,nl-1 do begin
	for x=0,ns-1 do begin
		t_dat=temp_data(x,y,*)
		t_mask=temp_mask(x,y,*)
		mittel, t_dat, t_mask, erg
		temp_out(x,y)=BYTE(erg)
	endfor
endfor
; ---------------------------------------------------------
; Bestimmung der realen Temperaturen
; ber Data Scaling & Kelvin Umrechnung
; ---------------------------------------------------------
temp_scaling=0.0200
kelvin=273.15
temp_out=temp_out * temp_scaling
temp_out=temp_out + kelvin
; ---------------------------------------------------------
; Ausgabe der gemittelten Datei
; ---------------------------------------------------------
openw,1,temp_name
writeu,1,temp_out
close,1
bnames=['Temperatur [C]']
ENVI_SETUP_HEAD, fname=temp_name, $
  ns=ns, nl=nl, nb=1, bnames=bnames,$
  interleave=0, data_type=4, $
  offset=0, map_info=map_info, $
  /write, r_fid=tmp_id
ENVI_FILE_MNG, id=tmp_id, /remove
; ---------------------------------------------------------
; Ende der Routine MOD11_TEMP_COMPOSIT
;----------------------------------------------------------
END
; *********************************************************
; *** MOD11_EMS_COMPOSIT **********************************
; *********************************************************
PRO MOD11_EMS_COMPOSIT, in_pfad, ems_outpfad
; ---------------------------------------------------------
; Massenprozessierung
; ---------------------------------------------------------
such_str=in_pfad+'\*Emissivity.modis'
data_name=FINDFILE(such_str, count=count)
tile_name=STRMID(data_name, (STRPOS( data_name(0), 'h',10 )),6)
use=WHERE(tile_name eq 'h19v10', count_use)
if count_use lt 1 then return
data_names=STRARR(count_use)
data_names=data_name(use)
count=count_use
; ---------------------------------------------------------
; Erzeugen des Ausgabenamens der EMS-Datei
; ---------------------------------------------------------
date=STRMID(data_names(0),(STRPOS(data_names(0),'\',/REVERSE_SEARCH)))
Prod=STRMID(date,(STRPOS(date,'M')),5)
year=STRMID(date,7,4)
period_hlp=STRMID(data_names(0),0,(STRPOS(data_names(0),'\',/REVERSE_SEARCH)))
period=STRMID(period_hlp,(STRPOS(period_hlp,'_',/REVERSE_SEARCH))+1,2)
ems_oname = 'TSP_EMS' + '_' + period + year + '.data'
ems_name = ems_outpfad+ems_oname
; ---------------------------------------------------------
; Einlesen von grundsaetzlichen Dateninformationen
; ---------------------------------------------------------
envi_open_file, data_names(0), r_fid=fid, /NO_REALIZE
envi_file_query, fid, ns=ns, nl=nl, nb=nb
dims = [-1, 0, ns-1, 0, nl-1]
map_info=envi_get_map_info(fid=fid)
envi_file_mng, id=fid, /remove
; ---------------------------------------------------------
; Erfassung der Emissivittsdaten
; ---------------------------------------------------------
ems_data=bytarr(ns,nl,count)
ems_mask=bytarr(ns,nl,count)
ems_temp=BYTARR(ns,nl)
emsb31_temp = BYTARR(ns,nl)
emsb32_temp = BYTARR(ns,nl)
for i=0,count-1 do begin
	; ---------------------------------------------------------
	; Einlesen der einzelnen Datei
	; ---------------------------------------------------------
	envi_open_file, data_names(i), r_fid=fid, /NO_REALIZE
		emsb31_temp(*,*)=ENVI_GET_DATA(fid=fid, dims=dims, pos=0)
		emsb32_temp(*,*)=ENVI_GET_DATA(fid=fid, dims=dims, pos=1)
	envi_file_mng, id=fid, /remove;, /delete
	print, 'EMS_Datei: ',i,' eingelesen.'
	; ************************************************
	; Ausmaskieren der No-Data-Bereiche
	; ************************************************
	b31_nodata=bytarr(ns,nl)
	no_data_ems, emsb31_temp, no_mask
	b31_nodata(*,*)=no_mask
	; ------------------------------------------------
   	b32_nodata=bytarr(ns,nl)
	no_data_ems, emsb32_temp, no_mask
	b32_nodata(*,*)=no_mask
	; ************************************************
	; Berechung der mittleren Emissivitt
	; ************************************************
	ems_dum=BYTARR(ns,nl)
	ems_dum(*,*)=0
	mask_dum=BYTARR(ns,nl)
	mask_dum(*,*)=0
	; -----------------------------------------------
	hilf=WHERE((b31_nodata(*,*) eq 0) and $
				(b32_nodata(*,*) eq 0),c_hilf)
	if c_hilf NE 0 then begin
		ems_dum(hilf)=BYTE(ROUND((emsb31_temp(hilf)+emsb32_temp(hilf))/2))
		mask_dum(hilf)=0
	endif
	; -----------------------------------------------
	hilf=WHERE((b31_nodata(*,*) eq 0) and $
				(b32_nodata(*,*) eq 1),c_hilf)
	if c_hilf NE 0 then begin
		ems_dum(hilf)=BYTE(ROUND(emsb31_temp(hilf)))
		mask_dum(hilf)=0
	endif
	; -----------------------------------------------
	hilf=WHERE((b31_nodata(*,*) eq 1) and $
				(b32_nodata(*,*) eq 0),c_hilf)
	if c_hilf NE 0 then begin
		ems_dum(hilf)=BYTE(ROUND(emsb32_temp(hilf)))
		mask_dum(hilf)=1
	endif
	; -----------------------------------------------
	hilf=WHERE((b31_nodata(*,*) eq 1) and $
				(b32_nodata(*,*) eq 1),c_hilf)
	if c_hilf NE 0 then begin
	    ems_dum(hilf)=0
		mask_dum(hilf)=1
	endif
	; -----------------------------------------------
	ems_data(*,*,i)=ems_dum
	ems_mask(*,*,i)=mask_dum
endfor
ems_dum=0
; ****************************************************
; Bestimmung der mittleren Temperatur ber 10 Tage
; *****************************************************
ems_out=BYTARR(ns,nl)
e_dat=INTARR(count)
e_mask=BYTARR(count)
for y=0,nl-1 do begin
	for x=0,ns-1 do begin
		e_dat=ems_data(x,y,*)
		e_mask=ems_mask(x,y,*)
		mittel, e_dat, e_mask, erg
		ems_out(x,y)=erg
	endfor
endfor
;----------------------------------------------------------
; Bestimmung der Emissivitt
; ber Data Scaling & Kelvin Umrechnung
;----------------------------------------------------------
ems_scaling = 0.00200
ems_offset = 0.4900
ems_out = (ems_out * ems_scaling) + ems_offset
;----------------------------------------------------------
; Ausgabe der gemittelten Datei
;----------------------------------------------------------
openw,1,ems_name
writeu,1,ems_out
close,1
bnames=['Emissivitt']
ENVI_SETUP_HEAD, fname=ems_name, $
  ns=ns, nl=nl, nb=1, bnames=bnames,$
  interleave=0, data_type=4, $
  offset=0, map_info=map_info, $
  /write, r_fid=ems_id
ENVI_FILE_MNG, id=ems_id, /remove
; ---------------------------------------------------------
; Ende der Routine MOD11_EMS_COMPOSIT.pro
; ---------------------------------------------------------
END
;**********************************************************
;PRO MOD11_PREP
;**********************************************************
; Die Routine MOD11_PREP.pro liest beliebige MODIS MOD11
; (Surface Temperature & Emissivity) ein und bereitet die
; allgemein fr die Anwendung im RBM Kaokoveld auf.
; Dabei bernimmt die Routine MOD11_PREP.pro eine steuernde
; Funktion und ruft die nachgeordneten Prozesse zur
; Datenverarbeitung auf.
; ---------------------------------------------------------
; VORDAFINIERTE VARIABLEN
; Diese sollen nur bei der Installation auf einer neuen
; Plattfrom angepasst werden !!
; ---------------------------------------------------------
tmp = 'D:\Temp'			; Pfad zu temporren Verzeichnis
						; zur Zwischenlagerung von Daten.
; ---------------------------------------------------------
; Eingangspfad
; ---------------------------------------------------------
pfad='E:\MOD11_2001_test\MOD11_test_2001.txt'
pix = 1000
; ---------------------------------------------------------
; Graphische Benutzeroberflche zur Eingabe der Pfade
; ---------------------------------------------------------
list = ['500','1000']
base=WIDGET_AUTO_BASE(title='RBM - MOD11 Data Preparation')
  w1=WIDGET_BASE(base, /row, /frame)
	e1=WIDGET_OUTF(w1, uvalue='ipfad', default=pfad, $
	   prompt='Please enter MOD11 directory file:', /auto)
    w3=WIDGET_BASE(base, /column, /frame)
   	p5 = WIDGET_OUTF(w3, uvalue='opfad', /directory, $
   	     prompt='Please enter output directory:', /auto)
	p6 = widget_toggle(w3, uvalue='pix', list=list, $
   	     prompt='Select import resolution [m]', /auto)
result=AUTO_WID_MNG(base)
if (result.accept eq 0) then RETALL  ;exit by cancel-button
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Auslesen der GUI
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
pfad=(result.ipfad)
;--------------------------
outf=(result.opfad)
pixel=(result.pix)
CASE pixel OF
0 :pix=500
1:pix=1000
ENDCASE
; ---------------------------------------------------------
; Definition der Ausgabe
; ---------------------------------------------------------
outf=outf+'\'
temp_outpfad = outf + 'Temperatur'
dirtest=FILE_TEST(temp_outpfad,/DIRECTORY)
 if dirtest EQ 0 THEN FILE_MKDIR, temp_outpfad
temp_outpfad=temp_outpfad+'\'
ems_outpfad = outf + 'Emissivity'
dirtest=FILE_TEST(ems_outpfad,/DIRECTORY)
 if dirtest EQ 0 THEN FILE_MKDIR, ems_outpfad
ems_outpfad=ems_outpfad+'\'
; ---------------------------------------------------------
; Bestimmung der Lnge der Pfaddatei
; ---------------------------------------------------------
A = ''		; Dummy-Variable zum Einlesen der Pfade
n = 0		; Zhlvariable z. Ermitt. d.Gesamtzahl d. Pfade
; ---------------------------------------------------------
; Schleife mit EOF Statment um Lnge zu Bestimmen
; ---------------------------------------------------------
OPENR,1,pfad
WHILE NOT EOF(1) DO BEGIN
 READF, 1, A
 n=n+1
ENDWHILE
CLOSE, 1
; ---------------------------------------------------------
; Echtes Einlesen der Pfadangaben
; ---------------------------------------------------------
pfade=STRARR(n)
OPENR,1,pfad
FOR i=0, n-1 DO BEGIN
 READF, 1, A 				; Read a line of text
 pfade(i) = A
ENDFOR
CLOSE, 1
; ---------------------------------------------------------
; B E G I N N   D E R   P R O Z E S S I E R U N G
; ---------------------------------------------------------
; 1. Import und Umprojektion aller einzelnen Daten
;    aus ISIN in UTM.
; ---------------------------------------------------------
; Erzeugung eines Ordner, der die Importprodukte aufnimmt
; ---------------------------------------------------------
pfad = STRMID(pfade,(STRPOS(pfade(0), '\', /REVERSE_SEARCH))+1)
import_outpfad=tmp +'\'+ pfad
; ---------------------------------------------------------
; Beginn der Importschleife
; ---------------------------------------------------------
FOR i=0,n-1 DO BEGIN
 ; berprfung des Ausgabeordners
 import_outpfd = import_outpfad(i)
 dirtest=FILE_TEST(import_outpfd,/DIRECTORY)
 if dirtest EQ 0 THEN FILE_MKDIR, import_outpfd
 import_outpfd = import_outpfd +'\'
; ---------------------------------------------------------
; Starten der Prozessierung
; ---------------------------------------------------------
  MOD11_HDF_IMPORT, pfade(i) , import_outpfd, pix, tmp
ENDFOR
; ---------------------------------------------------------
; 2. Zusammenfassen der tglichen Daten zu 10-Tages-
;    Kompositen
; ---------------------------------------------------------
import_outpfad=import_outpfad+'\'
for i=0,n-1 do begin
	; Starten der Prozessierung
	mod11_temp_composit, import_outpfad(i), temp_outpfad
endfor
for i=0,n-1 do begin
	; Starten der Prozessierung
	mod11_ems_composit, import_outpfad(i), ems_outpfad
endfor
; ---------------------------------------------------------
; Ende der Routine MOD11_PREP.pro
; ---------------------------------------------------------
PRINT, 'All Jobs finished !'
PRINT, 'Thanks for using MOD11_PREP'
PRINT, '(c) J.Richters, GIUB, Uni Bonn; 12/2003'
; ---------------------------------------------------------
; Ende der Routine MOD11_PREP.pro
; ---------------------------------------------------------
END