; Tool zur automatischen Umprojektion von MODIS Daten mit
; Hilfe des MODIS Reprojection Tool V.2.2
; ---------------------------------------------------------
; Das Programm arbeitet in folgender Reihenfolge:
; 1. 	Abfrage des Verzeichnisses, in dem die MODIS Rohdaten
;		liegen (Input) und Angabe des Ausgabeordners fr
;		prozessierten Daten.
; 2. 	bergabe der Dateinamen einzeln an die Subroutine
;		mod_reproject.pro. Hier wird der Ausgabedateiname
;		abgeleitet, die zugehrige Parameterdatei fr die
;		Projektion geschrieben und die C-Routine resample.exe
; 		gestartet. Anschlieend werden die erzeugten *.tif-
;		Dateien erneut in ENVI eingelesen und als ENVI-Datei
;		ausgegeben.
; 3.	Zusamenfgen der beiden zu einem Zeitpunkt gehrigen
;		Datenfiles ber die Mosaiking-Funktion von ENVI.
; *********************************************************
pro param, datname_in, tile_name, temp
; diese Programm erzeugt eine Parmeterfile um die C-Routine
; resample.exe zu steuern
; Erstellt am 28.02.2003 von J. Richters, RSRG - Uni Bonn
; Folgende Variablen werden von einer bergeordneten Routine
; bergeben:
; datname_in: Dies ist der Dateiname samt Pfad der MODIS
;	*.hdf Datei, die umprojetiert werden soll.(STRING Variable!)
parampfad=temp+'\project.prm'
datain='INPUT_FILENAME = '+datname_in
dataout='OUTPUT_FILENAME = '+temp+'modis_temp.tif'
;folgende mit 1 markierte Kanle werden prozessiert
P0='SPECTRAL_SUBSET = (1 0 0 0 0 1 0 0 0 1 0 0)'
P1=' '
P2='SPATIAL_SUBSET_TYPE = INPUT_LINE_SAMPLE'
case tile_name of
'h19v10': subset=[200,1300,2000,2400]
'h19v11': subset=[200,1,2000,1000]
endcase
uly=string(subset(0))
ulx=string(subset(1))
lry=string(subset(2))
lrx=string(subset(3))
P3='SPATIAL_SUBSET_UL_CORNER = ('+ulx+uly+' )'
P4='SPATIAL_SUBSET_LR_CORNER = ('+lrx+lry+' )'
P5=' '
P6='RESAMPLING_TYPE = nearest_neighbor'
P7=' '
P8='OUTPUT_PROJECTION_TYPE = UTM'
P9=' '
P10='UTM_ZONE = 33'
P11=' '
P12='OUTPUT_PIXEL_SIZE = 1000'
openw,1,parampfad
printf,1,datain
printf,1,P1
printf,1,dataout
printf,1,P1
printf,1,P0
printf,1,P1
printf,1,P2
printf,1,P3
printf,1,P4
printf,1,P5
printf,1,P6
printf,1,P7
printf,1,P8
printf,1,P9
printf,1,P10
printf,1,P11
printf,1,P12
printf,1,P1
close,1
end
;##########################################################
pro resample, datname_in, tile_name, temp
param, datname_in, tile_name, temp
; ---------------------------------------------------------
; Erzeugung einer Batchdatei zur Initialisierung
; der Umprojektion. Dabei muss die Shellvariable
; MRTDATADIR gesetzt werden.
; ---------------------------------------------------------
pre_command='set MRTDATADIR=D:\ENVI_IDL\startup\mrt31\data'
command='D:\ENVI_IDL\startup\mrt31\bin\resample -p '+temp+$
'project.prm -g ' +temp+'modis_resample.log'
; ---------------------------------------------------------
; Definition des Dateinamens und Ausgabe der Batchdatei
; ---------------------------------------------------------
bat_name='D:\ENVI_IDL\startup\mrt31\bin\ModisRun.bat'
OPENW,1,bat_name
PRINTF,1,pre_command
PRINTF,1,command
CLOSE,1
; ---------------------------------------------------------
; Absetzten des Shell-Befehls und bergabe des
; Befehls an die Kommandozeile
; ---------------------------------------------------------
SPAWN, bat_name
end
;##########################################################
; Modis_reproject.pro
;##########################################################
; diese Programm erzeugt projeziert mit Hilfe der C-Routine
; resample.exe MODIS Daten aus einer ISIN Integerized Sinusodial
; Grid in UTM, WGS84, Zone 33 South um.
; Erstellt am 28.02.2002 von J. Richters, RSRG - Uni Bonn
pro mod_project, datname_in, outpfad, temp
;Pfad zu dem temprr erzeugten Daten
temp_pfad=temp+'*.tif'
; Definition des Ausgabenamens
datname = STRMID(datname_in, (STRPOS( datname_in, 'MOD', $
/REVERSE_SEARCH)))
prod_name=STRMID(datname, 0,5)
datum_name=STRMID(datname, (STRPOS( datname, 'A',5 ))+1,7)
tile_name=STRMID(datname, (STRPOS( datname, 'h',10 )),6)
file_mkdir, outpfad+'reproj'
outname=outpfad +'reproj\'+ prod_name + '_' + datum_name + $
 '_' + tile_name + '.modis'
; Aufruf von resample.pro, die eine Steuerdatei fr die C-Routine
; erzeugt und dem Resamling Prozess anstt.
resample, datname_in, tile_name, temp
; Abfragen der Anzahl der erzeugten Dateien.
fname=findfile(temp_pfad, count=count)
if count lt 0 then return
; Oeffnen der ersten Datei um die Geometrie anzufragen
envi_open_data_file, fname(0), /tiff, r_fid=fid
  if (fid eq -1) then return
envi_file_query, fid, ns=ns, nl=nl, nb=nb
dims = [-1, 0, ns-1, 0, nl-1]
map_info=envi_get_map_info(fid=fid)
envi_file_mng, id=fid, /remove
; erzeugen eines Array, in dem die umprojezierten Daten gelagert werden.
data=bytarr(ns,nl,count)
; Einlesen einzelen Bnder und ausgabe in Speicherarray
for i=0,count-1 do begin
envi_open_data_file, fname(i), r_fid=fid, /tiff
data(*,*,i)=fix(ENVI_GET_DATA(fid=fid, dims=dims, pos=0))
envi_file_mng, id=fid, /remove, /delete
endfor
bnames=['Bare [%]','Nontree Vegetation [%]','Tree Cover [%]']
; Ausgabe der Umprojezierten Datei
openw,2,outname
writeu, 2, data
close, 2
ENVI_SETUP_HEAD, fname=outname, $
  ns=ns, nl=nl, nb=count, bnames=bnames, $
  interleave=0, data_type=1, $
  offset=0, map_info=map_info, $
  /write, r_fid=out_id
envi_file_mng, id=out_id, /remove
end
;##########################################################
; Definition eines temporren Verzeichnisses
;**********************************************************
temp='d:\temp\MODIS'; Temporres Verzeichnis zur Zwischen-
					; lagerung von Daten
; berprfung der Temporren Ordner
dirtest=FILE_TEST(temp,/DIRECTORY)
	if dirtest eq 0 then FILE_MKDIR, 'd:\temp\MODIS'
temp=temp+'\'
;**********************************************************
; Eingabename der MODIS +.hdf Datei
;**********************************************************
in_pfad=envi_pickfile(title='Whlen Sie die erste *.HDF Datei:', $
						filter='*.hdf')
if (in_pfad eq '') then retall
datadir = STRMID(in_pfad, 0,(STRPOS(in_pfad, '\', /REVERSE_SEARCH))+1)
such_str=datadir+'*.hdf'
data_names=findfile(such_str, count=filezahl)
if filezahl lt 1 then retall
;**********************************************************
; GUI um Prozessierung zu berwachen und Ausgabefestzulegen.
;**********************************************************
base =widget_auto_base(title='RBM - MODIS Processing', /XBIG)
	datlist  = widget_slist(base, prompt='Use following files for processing',$
				list=data_names, /frame, /no_select)
	infotext='No. of selected files: '+strtrim(string(filezahl),1)
	Info = WIDGET_SLABEL(Base, prompt=infotext)
	wo = widget_outf(base, uvalue='outf',  /directory , $
	prompt='Please Select Output Directory',/auto)
result = auto_wid_mng(base)
if (result.accept eq 0) then retall
outpfad=result.outf
;**********************************************************
; Beginn der Prozessierung
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; 1. Umprojezieren der MODIS *.hdf Daten aus SINUSODIAL in
; UTM, WGS84
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	for i=0, filezahl-1 do begin
	mod_project, data_names(i), outpfad, temp
	endfor
;**********************************************************
; Ende der Prozessierung erreicht
;**********************************************************
print,'End of Processing reached'
print,'Thanks for using MOD44_PREP.pro'
end