; RBM_PREP.pro (v0.1)
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Diese Routine wurde von J: Richters am 09.10.2003
; im Rahmen des Teilprojekt B7 im SFB 389 ACACIA
; am GIUB erstellt
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;   Die Routine RBM_PREP.pro ist die zwingende Vorbereitung
; fr einen erfolgreichen Verlauf einer Biomassenberechnung
; mit RBM_KAOKOVELD_ALL.pro. Diese vorbereitende Routine
; analsiert, ordnet und testet alle Dateien, die fr einen
; erfolgreichen Modellauf ntig sind. Dazu werden alle
; bentigten Dateien aufgerufen, ggf. beschnitten und in
; die modelleigene Ordner und Dateistruktur geschreiben.
;   Die Arbeit der Routine RBM_PREP.pro unfasst folgende
; Schritte:
; - Abfrage von Benutzerinformatioen (Home-Verzeichnis,
; 	Zeitraum des Modelllaufs, rumliche Eingrenzung).
; - Abfrage der Modelleingangsdaten (hier werden jeweils
;	die Pfade zu den Verzeichnissen erhoben, da meist
;	jeweils 1 Jahr komplett pro Zeitschritt vorliegt).
; - Aus den erhobene Daten werden nun smtliche Ordner
;	angelegt, die fr die Funktion des Modells von
;	Bedeutung sind.
; - Danach werden alle bentigten Dateien geffnet,
;	rumlich beschnitten und dann in der modelleigene
;	Ornerstruktur abgelegt.
; - Parallel dazu wird eine Logfile erstellt, die alle
;	Vorkommnisse protokolliert. Erst wenn hier alle
;	Dateien als korrekt verzeichnet sind, kann der
;	Modelllauf ber die Routine RBM_KAOKOVELD_ALL.pro
;	gestartet werden.
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; PRO SUBSET_SINGLE.pro
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Diese Routine wurde von J. Richters
; am 08.10.2003 im Rahmen des SFB 389 - ACACIA,
; Teilprojekt B7 am GIUB erstellt.
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Diese Routine schneidet ein Subset aus der angelieferten
; Daten. Dazu sind lediglich folgende Parameter ntig:
; - Eingabepfad
; - Ausgabepfad
; - UL_xy   		Obere Linke Ecke des Subsets
; - LR_xy			Untere Rechte Ecke des Subsets
; Zustzlich liefert die Routine noch eine Log-Meldung
; ber den Erfolg oder Misserfolg der Dateioperation
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PRO SUBSET_SINGLE, data_in, data_out, UL_xy, LR_xy, ns, nl, log
result=FILE_TEST(data_in)
  IF (result NE 1) THEN BEGIN
  log='ERROR: File ' + data_out + ' not written: File ' + $
  	  data_in + ' not found'
  RETURN
  ENDIF
ENVI_OPEN_FILE, data_in, r_fid=fid, /no_realize
; Anpassung der Eckkoordinaten
xmap=[UL_xy(0),LR_xy(0)]
ymap=[UL_xy(1),LR_xy(1)]
; Transformation der Map-Koordinaten in File-Pixel
ENVI_CONVERT_FILE_COORDINATES, fid, xf, yf, xmap, ymap
xf=ROUND(xf)
yf=ROUND(yf)
; Ruecktransformation der File-Pixel in Map-Koordinaten um
; Fehler aus Integerwandlung auszugleichen
ENVI_CONVERT_FILE_COORDINATES, fid, xf, yf, x_map, y_map, /to_map
IF (ns EQ 0) THEN BEGIN
  if xf(0) gt xf(1) then ns=(xf(0)-xf(1))+1 $
    else ns=(xf(1)-xf(0))+1
ENDIF
IF (nl EQ 0) THEN BEGIN
  if yf(0) gt yf(1) then nl=(yf(0)-yf(1))+1 $
    else nl=(yf(1)-yf(0))+1
ENDIF
ENVI_FILE_QUERY, fid, nb=nb, data_type=dt, bnames=bnames
dims = [-1, xf(0), xf(0)+ns-1, yf(0), yf(0)+nl-1]
pos=INTARR(nb)
map_info = ENVI_GET_MAP_INFO(fid=fid)
map_info.mc(2)=x_map(0)
map_info.mc(3)=y_map(0)

if (dt eq 1) then data=BYTARR(ns,nl,nb)
if (dt eq 2) then data=INTARR(ns,nl,nb)
if (dt eq 3) then data=LONARR(ns,nl,nb)
if (dt eq 4) then data=FLTARR(ns,nl,nb)
if (dt eq 5) then data=DBLARR(ns,nl,nb)
if (dt eq 6) then data=DCOMPLEXARR(ns,nl,nb)
if (dt eq 14) then data=LONARR(ns,nl,nb)
if (dt eq 12) then data=UINTARR(ns,nl,nb)
if (dt eq 13) then data=ULONARR(ns,nl,nb)
if (dt eq 15) then data=ULON64ARR(ns,nl,nb)

for i=0, nb-1 do begin
data(*,*,i)=ENVI_GET_DATA(fid=fid, dims=dims, pos=i)
endfor
ENVI_FILE_MNG, id=fid, /remove ;,/delete
; Ausgabe der kopierten Datei
OPENW, 10, data_out
WRITEU,10, data
CLOSE,10
ENVI_SETUP_HEAD, fname=data_out, $
  ns=ns, nl=nl, nb=nb, bnames=bnames, $
  interleave=0, data_type=dt, $
  offset=0, file_type=0, $
  map_info=map_info, /write, r_fid=oid
ENVI_FILE_MNG, id=oid, /remove ;,/delete
log='OK: File ' + data_out + ' written'
END
; #########################################################
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ++++++++ M A I N   P R O C E S S I N G   L E V E L ++++++
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PRO RBM_PREP_GUI
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Meldung des Programms
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
print, 'This is RBM_PREP.pro :: Date ', SYSTIME()
print,'(C) J.Richters, 10/2003; SFB389 - ACACIA TP B7; built at GIUB'
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Einstellungen
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Pfad='E:'
run_name='RBM_test_03'
HOME_PFD= Pfad+'\'+run_name
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Zeitliche Eingrenzung
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
modelling_start=	[1,2001]
modelling_end=		[36,2001]
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Rumliche Eingrenzung
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
north_border=-17.0
east_border=14.0
south_border=-18.0
west_border=13.0
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Angabe der unvernderlichen Modellparameter
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
utm_zone=33
datum='WGS84'
ns=0
nl=0
TS_pa=36		; Angabe der Modellzeitschritte pro Jahr (= 36)
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Dialogfenster zur Eingabe der grundstzlichen
; Modellparameter
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
base=WIDGET_AUTO_BASE(title='RBM - Modelling Preparation')
  w1=WIDGET_BASE(base, /row, /frame)
	e1=WIDGET_OUTF(w1, uvalue='home', /directory, $
	   prompt='Please enter modelling base directory',$
	   default=HOME_PFD, /auto)
  w2=WIDGET_BASE(base, column=2, /frame)
   	p1 = WIDGET_PARAM(w2, dt=4, field=8, xsize=10, $
         default=north_border, uvalue='north', $
         prompt='North Border', /auto)
   	p2 = WIDGET_PARAM(w2, dt=4, field=8, xsize=10, $
         default=east_border, uvalue='east', $
         prompt='East Border', /auto)
   	p3 = WIDGET_PARAM(w2, dt=4, field=8, xsize=10, $
         default=south_border, uvalue='south', $
         prompt='South Border', /auto)
   	p4 = WIDGET_PARAM(w2, dt=4, field=8, xsize=10, $
         default=west_border, uvalue='west', $
         prompt='West Border', /auto)
  w3=WIDGET_BASE(base, row=2, /frame)
   	p5 = WIDGET_PARAM(w3, dt=2, xsize=2, $
         default=modelling_start(0), uvalue='start_ts', $
         prompt='Modelling Start: Timestep', /auto)
   	p6 = WIDGET_PARAM(w3, dt=2, xsize=4, $
         default=modelling_start(1), uvalue='start_year', $
         prompt='Year', /auto)
   	p7 = WIDGET_PARAM(w3, dt=2, xsize=2, $
         default=modelling_end(0), uvalue='end_ts', $
         prompt='Modelling End: Timestep', /auto)
   	p8 = WIDGET_PARAM(w3, dt=2, xsize=4, $
         default=modelling_end(1), uvalue='end_year', $
         prompt='Year', /auto)
result=AUTO_WID_MNG(base)
if (result.accept eq 0) then RETALL  ;exit by cancel-button
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Auslesen der ersten GUI
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
HOME_PFD=(result.home)
;--------------------------
north_border=(result.north)
east_border=(result.east)
south_border=(result.south)
west_border=(result.west)
;--------------------------
modelling_start(0)=(result.start_ts)
modelling_start(1)=(result.start_year)
modelling_end(0)=(result.end_ts)
modelling_end(1)=(result.end_year)
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Berechung der Modelllaufzeit
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der Modelljahre
model_years=modelling_end(1) - modelling_start(1)+1
y=0
model_time=INTARR(model_years,2)
for t=0,model_years-1 do begin
	model_time(t,0)=modelling_start(1)+y
	y=y+1
endfor
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ermittlung der Modellzeitschritte
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
model_time(0,1)= TS_pa - modelling_start(0) +1
FOR t=1,model_years-2 DO BEGIN
	model_time(t,1)=TS_pa
ENDFOR
model_time(model_years -1,1)=modelling_end(0)
modelling_time=STRTRIM(model_time,2)
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Auswahl der Pfade pro Jahr (TSP-Produkte)
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ALL_PFD=STRARR(model_years,6)
FOR t=0, model_years-1 DO BEGIN
	;++++++++++++++++++++++++++++++++++++++++++++++++++++++
	; Aufbau der GUI zur Erfassung der jhrlichen Daten
	;++++++++++++++++++++++++++++++++++++++++++++++++++++++
	PFD_title='RBM - Modelling Data Paths - Year: ' + $
			  modelling_time(t,0)
	e1_prompt='Path to Spectral Refl.Data [Year:' + $
			   modelling_time(t,0) + ']:'
	e2_prompt='Path to Cloud Data [Year:' + $
			   modelling_time(t,0) + ']:'
	e3_prompt='Path to Surface Temperature Data [Year:' + $
			   modelling_time(t,0) + ']:'
	e4_prompt='Path to Surface Emissivity Data [Year:' + $
			   modelling_time(t,0) + ']:'
	e5_prompt='Path to Water Vapor Data [Year:' + $
			   modelling_time(t,0) + ']:'
	e6_prompt='Please select Vegetation Continuous ' + $
			   'Fields Data [Year:' + $
			   modelling_time(t,0) + ']:'
	base_tsp=WIDGET_AUTO_BASE(title=PFD_title)
	  w2=WIDGET_BASE(base_tsp, /column, /frame)
		e1=WIDGET_OUTF(w2, uvalue='mod09', /directory, $
		   prompt=e1_prompt, /auto)
		e2=WIDGET_OUTF(w2, uvalue='cloud', /directory, $
		   prompt=e2_prompt, /auto)
		e3=WIDGET_OUTF(w2, uvalue='temp', /directory, $
		   prompt=e3_prompt, /auto)
		e4=WIDGET_OUTF(w2, uvalue='ems', /directory, $
		   prompt=e4_prompt, /auto)
		e5=WIDGET_OUTF(w2, uvalue='wv', /directory, $
		   prompt=e5_prompt, /auto)
		e6=WIDGET_OUTF(w2, uvalue='vcf', $
		   prompt=e6_prompt, /auto)
	pfd_result=AUTO_WID_MNG(base_tsp)
	IF (pfd_result.accept EQ 0) THEN RETALL  ;exit by cancel-button
	;++++++++++++++++++++++++++++++++++++++++++++++++++++++
	; Auslesen der GUI fr jhrliche Daten
	;++++++++++++++++++++++++++++++++++++++++++++++++++++++
	ALL_PFD(t,0)=(pfd_result.mod09)
	ALL_PFD(t,1)=(pfd_result.cloud)
	ALL_PFD(t,2)=(pfd_result.temp)
	ALL_PFD(t,3)=(pfd_result.ems)
	ALL_PFD(t,4)=(pfd_result.wv)
	ALL_PFD(t,5)=(pfd_result.vcf)
ENDFOR
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Auswahl der Pfade pro Modellauf (CDP)
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
CDP_PFD=STRARR(2)
CDP_title='RBM - Modelling Data Paths - ' + $
		  'Constant Data Products'
e1_prompt='Path to Digtal Elevation Model [DEM]'
e2_prompt='Path to FAO Soil Map [SOIL]'
e3_prompt='Path to Starting NPP-Data [NPP_balance]'
base_cdp=WIDGET_AUTO_BASE(title=CDP_title)
  w2=WIDGET_BASE(base_cdp, /column, /frame)
	e1=WIDGET_OUTF(w2, uvalue='dem', $
	   prompt=e1_prompt, /auto)
	e2=WIDGET_OUTF(w2, uvalue='soil', $
	   prompt=e2_prompt, /auto)
	e3=WIDGET_OUTF(w2, uvalue='NPP_bal', $
	   prompt=e3_prompt, /auto)
cdp_result=AUTO_WID_MNG(base_cdp)
; ---------------------------------------------------------
; Auslesen der GUI fr konstante Datenpfade
; ---------------------------------------------------------
CDP_PFD(0)=(cdp_result.dem)
CDP_PFD(1)=(cdp_result.soil)
NPP_PFD = cdp_result.NPP_bal
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Anlage des Modellierungsverzeichnisses
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Matrix mit Pfadnamen anlegen
MODEL_PFD=STRARR(model_years,13)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Prfung des Homeverzeichnis
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
home_test = FILE_TEST(HOME_PFD, /DIRECTORY)
IF (home_test EQ 1) THEN BEGIN
GOTO, end_home
ENDIF ELSE BEGIN
FILE_MKDIR, HOME_PFD
ENDELSE
end_home:
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Prfung des Balance-Verzeichnis
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BAL_PFD=HOME_PFD + '\balance'
bal_test = FILE_TEST(BAL_PFD, /DIRECTORY)
IF (bal_test EQ 1) THEN GOTO, level2 $
ELSE FILE_MKDIR, BAL_PFD
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Prfung und Anlage der 2. Ordnerlevels
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
level2:
for t=0,N_ELEMENTS(modelling_time(*,0))-1 do begin
; -----------------------------------------------
	l2_pfd=HOME_PFD+'\'+modelling_time(t,0)
	result = FILE_TEST(l2_pfd, /DIRECTORY)
	if (result eq 1) then goto, level3_1 $
	 else FILE_MKDIR, l2_pfd
	level3_1:
; -----------------------------------------------
	l3_1_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\ysp'
	result = FILE_TEST(l3_1_pfd, /DIRECTORY)
	if (result eq 1) then goto, level3_2 $
	 else FILE_MKDIR, l3_1_pfd
	level3_2:
	MODEL_PFD(t,0)= l3_1_pfd
; -----------------------------------------------
	MODEL_PFD(t,0)= l3_1_pfd
	l3_2_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp'
	result = FILE_TEST(l3_2_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_1 $
	 else FILE_MKDIR, l3_2_pfd
	level4_1:
; -----------------------------------------------
	l4_1_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\mod09'
	result = FILE_TEST(l4_1_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_2 $
	 else FILE_MKDIR, l4_1_pfd
	level4_2:
	MODEL_PFD(t,1)= l4_1_pfd
; -----------------------------------------------
	l4_2_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\ndvi'
	result = FILE_TEST(l4_2_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_3 $
	 else FILE_MKDIR, l4_2_pfd
	level4_3:
	MODEL_PFD(t,2)= l4_2_pfd
; -----------------------------------------------
	l4_3_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\temperatur'
	result = FILE_TEST(l4_3_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_4 $
	 else FILE_MKDIR, l4_3_pfd
	level4_4:
	MODEL_PFD(t,3)= l4_3_pfd
; -----------------------------------------------
	l4_4_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\emissivity'
	result = FILE_TEST(l4_4_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_5 $
	 else FILE_MKDIR, l4_4_pfd
	level4_5:
	MODEL_PFD(t,4)= l4_4_pfd
; -----------------------------------------------
	l4_5_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\watervapor'
	result = FILE_TEST(l4_5_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_6 $
	 else FILE_MKDIR, l4_5_pfd
	level4_6:
	MODEL_PFD(t,5)= l4_5_pfd
; -----------------------------------------------
	l4_6_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\cloud'
	result = FILE_TEST(l4_6_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_7 $
	 else FILE_MKDIR, l4_6_pfd
	level4_7:
	MODEL_PFD(t,6)= l4_6_pfd
; -----------------------------------------------
	l4_7_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\biomass'
	result = FILE_TEST(l4_7_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_8 $
	 else FILE_MKDIR, l4_7_pfd
	level4_8:
	MODEL_PFD(t,7)= l4_7_pfd
; -----------------------------------------------
	l4_8_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\par'
	result = FILE_TEST(l4_8_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_9 $
	 else FILE_MKDIR, l4_8_pfd
	level4_9:
	MODEL_PFD(t,8)= l4_8_pfd
; -----------------------------------------------
	l4_9_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\par_eff'
	result = FILE_TEST(l4_9_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_10 $
	 else FILE_MKDIR, l4_9_pfd
	level4_10:
	MODEL_PFD(t,9)= l4_9_pfd
; -----------------------------------------------
	l4_10_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\BCF_eff'
	result = FILE_TEST(l4_10_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_11 $
	 else FILE_MKDIR, l4_10_pfd
	level4_11:
	MODEL_PFD(t,10)= l4_10_pfd
; -----------------------------------------------
	l4_11_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\PET'
	result = FILE_TEST(l4_11_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_12 $
	 else FILE_MKDIR, l4_11_pfd
	level4_12:
	MODEL_PFD(t,11)= l4_11_pfd
; -----------------------------------------------
	l4_12_pfd=HOME_PFD+'\'+modelling_time(t,0)+'\tsp\FPAR'
	result = FILE_TEST(l4_12_pfd, /DIRECTORY)
	if (result eq 1) then goto, level4_13 $
	 else FILE_MKDIR, l4_12_pfd
	level4_13:
	MODEL_PFD(t,12)= l4_12_pfd
; -----------------------------------------------
ENDFOR
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Beginn der Datenprozessierung
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Bestimmung des Datensubsets (Map Coordinates)
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
iproj = ENVI_PROJ_CREATE(/geographic)
if (north_border lt 0) then begin
oproj = ENVI_PROJ_CREATE(/utm,zone=utm_zone, /south, datum =datum)
endif else begin
oproj = ENVI_PROJ_CREATE(/utm,zone=utm_zone, datum=datum)
endelse
; Create the pairs of Latitude and Longitude
ixmap = [east_border, west_border]
iymap =   [north_border, south_border]
; Convert from Geographic to UTM
ENVI_CONVERT_PROJECTION_COORDINATES, $
  ixmap, iymap, iproj, $
  oxmap, oymap, oproj
; Sorting of the UTM pairs
UL_xy=[oxmap(1), oymap(0)]	;Upper Left X in Map Coordinates
							;Upper Left Y in Map Coordinates
LR_xy=[oxmap(0), oymap(1)]	;Lower Right X in Map Coordinates
							;Lower Right Y in Map Coordinates
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Erstellung einer Log Datei, die Programmmeldungen
; aufnimmt und als Textfile speichert
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
LOG_PFD=HOME_PFD + '\RBM_Prep_file_0.log'
;----------------------------------------------------------
; Prfung der Log Datei:
; wenn schon existent, neuer Dateiname
;----------------------------------------------------------
result=FILE_TEST(LOG_PFD)
   if (result eq 1) then begin
    Log_such=HOME_PFD + '\*.log'
	logs=FINDFILE(Log_such, count=log_c)
	log_nr=FIX(STRMID(logs(log_c-1), $
	       STRPOS(logs(log_c-1),'_',/REVERSE_SEARCH)+1, $
	       STRPOS(logs(log_c-1),'.',/REVERSE_SEARCH)))
	LOG_PFD=HOME_PFD + '\RBM_PREP_file_' + $
		    STRTRIM((log_nr+1),2) + '.log
   endif
;----------------------------------------------------------
; Beginn der Log Datei
;----------------------------------------------------------
OPENW,1, LOG_PFD
PRINTF,1, 'This is RBM Enironment Modelling Preparation Log '
PRINTF,1, 'written in ENVI/IDL 3.5'
PRINTF,1, 'by J. Richters (C)2003'
PRINTF,1, 'This RBM Environment in running under:'
PRINTF,1, !VERSION
PRINTF,1, ''
PRINTF,1, 'Check of written files:'
PRINTF,1, 'Begin of Logging  ', SYSTIME()
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Verarbeitung einmaliger Daten
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Ablage der allg. Modellparameter
; ---------------------------------------------------------
TIME_PFD=HOME_PFD + '\rbm_pref.rbm'
MODEL_TIM = STRARR(4)
MODEL_TIM(0)=STRTRIM(modelling_start(0),2)
MODEL_TIM(1)=STRTRIM(modelling_start(1),2)
MODEL_TIM(2)=STRTRIM(modelling_end(0),2)
MODEL_TIM(3)=STRTRIM(modelling_end(1),2)
OPENW,10,TIME_PFD
PRINTF,10,'MOD_TIME_Start_TS='+MODEL_TIM(0)
PRINTF,10,'MOD_TIME_Start_YR='+MODEL_TIM(1)
PRINTF,10,'MOD_TIME_END_TS='+MODEL_TIM(2)
PRINTF,10,'MOD_TIME_END_YR='+MODEL_TIM(3)
CLOSE,10
;----------------------------------------------------------
; Log Meldung
;----------------------------------------------------------
PRINTF,1, 'File: '+TIME_PFD+' written.'
;----------------------------------------------------------
; Ablage der NPP-Balance
; ---------------------------------------------------------
npp_name='BIOMASS_ALL.data'
npp_out=BAL_PFD+'\'+npp_name
subset_single, NPP_PFD, npp_out, UL_xy, LR_xy, ns, nl, log
;----------------------------------------------------------
; Log Meldung
;----------------------------------------------------------
PRINTF,1, log
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Verarbeitung der Daten auf jhrlicher Basis
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
FOR t=0,N_ELEMENTS(model_time(*,0))-1 DO BEGIN
;------------------------------------------------------
; Log Meldung
;------------------------------------------------------
PRINTF,1, 'Year: ',STRTRIM(model_time(t,0))
; -----------------------------------------------------
; Verarbeitung und Ablage der YSP Daten: DGM
; -----------------------------------------------------
dgm_name='CDP_DGM_'+modelling_time(t,0)+'.data'
dgm_out=MODEL_PFD(t,0)+'\'+dgm_name
subset_single, CDP_PFD(0), dgm_out, UL_xy, LR_xy, ns, nl, log
;------------------------------------------------------
; Log Meldung
;------------------------------------------------------
PRINTF,1, log
; -----------------------------------------------------
; Verarbeitung und Ablage der YSP Daten: Soil
; -----------------------------------------------------
soil_name='CDP_SOIL_'+modelling_time(t,0)+'.data'
soil_out=MODEL_PFD(t,0)+'\' + soil_name
subset_single, CDP_PFD(1), soil_out, UL_xy, LR_xy,ns, nl, log
;------------------------------------------------------
; Log Meldung
;------------------------------------------------------
PRINTF,1, log
; -----------------------------------------------------
; Verarbeitung Ablage der YSP Daten: VCF
; -----------------------------------------------------
VCF_name='YDP_VCF_'+modelling_time(t,0)+'.data'
VCF_out=MODEL_PFD(t,0)+'\' + VCF_name
subset_single, ALL_PFD(t,5), VCF_out, UL_xy, LR_xy, ns, nl, log
;------------------------------------------------------
; Log Meldung
;------------------------------------------------------
PRINTF,1, log
; -----------------------------------------------------
; Verarbeitung Ablage der YSP Daten: XXX
; -----------------------------------------------------
; +++++++++++++++++++++++++++++++++++++++++++++++++++++
; Verarbeitung und Ablage der TSP Daten: MOD09
; +++++++++++++++++++++++++++++++++++++++++++++++++++++
; Prfung der Eingangsdaten: MOD09
; ----------------------------------------------------------
such_str=ALL_PFD(t,0)+'*.data'
names=FINDFILE(such_str, count=count)
data_names= STRMID(names,(STRPOS(names, '\', /REVERSE_SEARCH))+1)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Konstruktion des Dateinamens
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
data_hilf=STRARR(TS_pa)
zr=0
for TS=0,8 do begin
	zr=zr+1
	data_hilf(TS)='TSP_MOD09_0'+STRTRIM(zr,2)+ $
				  STRTRIM(model_time(t,0),2)+'.data'
endfor
for TS=9,TS_pa-1 do begin
	zr=zr+1
	data_hilf(TS)='TSP_MOD09_'+STRTRIM(zr,2)+ $
				  STRTRIM(model_time(t,0),2)+'.data'
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;Abfangen von Anfangs- und ...
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (t eq 0) then begin
data_out=STRARR(model_time(t,1))
	j=0
	for i=modelling_start(0)-1,TS_pa-1 do begin
		data_out(j)=data_hilf(i)
		j=j+1
	endfor
endif
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ...Endverlusten.
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (t eq (model_years-1)) then begin
data_out=STRARR(model_time(t,1))
	for i=0, modelling_end(0)-1 do begin
		data_out(i)=data_hilf(i)
	endfor
endif
; ---------------------------------------------------------
; Verarbeitung und Ablage der YSP Daten: MOD09
; ---------------------------------------------------------
for i=0,(N_ELEMENTS(data_out))-1 do begin
	data_in=ALL_PFD(t,0)+'\'+data_out(i)
	dat_out=MODEL_PFD(t,1)+'\'+data_out(i)
	subset_single, data_in, dat_out, UL_xy, LR_xy, ns, nl, log
	;------------------------------------------------------
	; Log Meldung
	;------------------------------------------------------
	PRINTF,1, log
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++
; Verarbeitung und Ablage der TSP Daten: TEMP
; +++++++++++++++++++++++++++++++++++++++++++++++++++++
; Prfung der Eingangsdaten: TEMP
; ----------------------------------------------------------
such_str=ALL_PFD(t,2)+'*.data'
names=FINDFILE(such_str, count=count)
data_names= STRMID(names,(STRPOS(names, '\', /REVERSE_SEARCH))+1)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Konstruktion des Dateinamens
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
data_hilf=STRARR(TS_pa)
zr=0
for TS=0,8 do begin
	zr=zr+1
	data_hilf(TS)='TSP_TEMP_0'+STRTRIM(zr,2)+ $
				  STRTRIM(model_time(t,0),2)+'.data'
endfor
for TS=9,TS_pa-1 do begin
	zr=zr+1
	data_hilf(TS)='TSP_TEMP_'+STRTRIM(zr,2)+ $
				  STRTRIM(model_time(t,0),2)+'.data'
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;Abfangen von Anfangs- und ...
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (t eq 0) then begin
data_out=STRARR(model_time(t,1))
	j=0
	for i=modelling_start(0)-1,TS_pa-1 do begin
		data_out(j)=data_hilf(i)
		j=j+1
	endfor
endif
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ...Endverlusten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (t eq (model_years-1)) then begin
data_out=STRARR(model_time(t,1))
	for i=0, modelling_end(0)-1 do begin
		data_out(i)=data_hilf(i)
	endfor
endif
; ---------------------------------------------------------
; Verarbeitung und Ablage der YSP Daten: TEMP
; ---------------------------------------------------------
for i=0,(N_ELEMENTS(data_out))-1 do begin
	data_in=ALL_PFD(t,2)+'\'+data_out(i)
	dat_out=MODEL_PFD(t,3)+'\'+data_out(i)
	subset_single, data_in, dat_out, UL_xy, LR_xy, ns, nl, log
	;------------------------------------------------------
	; Log Meldung
	;------------------------------------------------------
	PRINTF,1, log
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++
; Verarbeitung und Ablage der TSP Daten: CLOUD
; +++++++++++++++++++++++++++++++++++++++++++++++++++++
; Prfung der Eingangsdaten: CLOUD
; ----------------------------------------------------------
such_str=ALL_PFD(t,1)+'*.data'
names=FINDFILE(such_str, count=count)
data_names= STRMID(names,(STRPOS(names, '\', /REVERSE_SEARCH))+1)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Konstruktion des Dateinamens
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
data_hilf=STRARR(TS_pa)
zr=0
for TS=0,8 do begin
	zr=zr+1
	data_hilf(TS)='TSP_CLD_0'+STRTRIM(zr,2)+ $
				  STRTRIM(model_time(t,0),2)+'.data'
endfor
for TS=9,TS_pa-1 do begin
	zr=zr+1
	data_hilf(TS)='TSP_CLD_'+STRTRIM(zr,2)+ $
				  STRTRIM(model_time(t,0),2)+'.data'
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;Abfangen von Anfangs- und ...
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (t eq 0) then begin
data_out=STRARR(model_time(t,1))
	j=0
	for i=modelling_start(0)-1,TS_pa-1 do begin
		data_out(j)=data_hilf(i)
		j=j+1
	endfor
endif
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ...Endverlusten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (t eq (model_years-1)) then begin
data_out=STRARR(model_time(t,1))
	for i=0, modelling_end(0)-1 do begin
		data_out(i)=data_hilf(i)
	endfor
endif
; ---------------------------------------------------------
; Verarbeitung und Ablage der YSP Daten: CLOUD
; ---------------------------------------------------------
for i=0,(N_ELEMENTS(data_out))-1 do begin
	data_in=ALL_PFD(t,1)+'\'+data_out(i)
	dat_out=MODEL_PFD(t,6)+'\'+data_out(i)
	subset_single, data_in, dat_out, UL_xy, LR_xy, ns, nl, log
	;------------------------------------------------------
	; Log Meldung
	;------------------------------------------------------
	PRINTF,1, log
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++
; Verarbeitung und Ablage der TSP Daten: WATERVAPOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++
; Prfung der Eingangsdaten: WATERVAPOR
; ----------------------------------------------------------
such_str=ALL_PFD(t,4)+'*.data'
names=FINDFILE(such_str, count=count)
data_names= STRMID(names,(STRPOS(names, '\', /REVERSE_SEARCH))+1)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Konstruktion des Dateinamens
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
data_hilf=STRARR(TS_pa)
zr=0
for TS=0,8 do begin
	zr=zr+1
	data_hilf(TS)='TSP_WV_0'+STRTRIM(zr,2)+ $
				  STRTRIM(model_time(t,0),2)+'.data'
endfor
for TS=9,TS_pa-1 do begin
	zr=zr+1
	data_hilf(TS)='TSP_WV_'+STRTRIM(zr,2)+ $
				  STRTRIM(model_time(t,0),2)+'.data'
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;Abfangen von Anfangs- und ...
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (t eq 0) then begin
data_out=STRARR(model_time(t,1))
	j=0
	for i=modelling_start(0)-1,TS_pa-1 do begin
		data_out(j)=data_hilf(i)
		j=j+1
	endfor
endif
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ...Endverlusten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (t eq (model_years-1)) then begin
data_out=STRARR(model_time(t,1))
	for i=0, modelling_end(0)-1 do begin
		data_out(i)=data_hilf(i)
	endfor
endif
; ---------------------------------------------------------
; Verarbeitung und Ablage der YSP Daten: WATERVAPOR
; ---------------------------------------------------------
for i=0,(N_ELEMENTS(data_out))-1 do begin
	data_in=ALL_PFD(t,4)+'\'+data_out(i)
	dat_out=MODEL_PFD(t,5)+'\'+data_out(i)
	subset_single, data_in, dat_out, UL_xy, LR_xy, ns, nl, log
	;------------------------------------------------------
	; Log Meldung
	;------------------------------------------------------
	PRINTF,1, log
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++
; Verarbeitung und Ablage der TSP Daten: EMISSIVITY
; +++++++++++++++++++++++++++++++++++++++++++++++++++++
; Prfung der Eingangsdaten: EMISSIVITY
; ----------------------------------------------------------
such_str=ALL_PFD(t,3)+'*.data'
names=FINDFILE(such_str, count=count)
data_names= STRMID(names,(STRPOS(names, '\', /REVERSE_SEARCH))+1)
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Konstruktion des Dateinamens
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
data_hilf=STRARR(TS_pa)
zr=0
for TS=0,8 do begin
	zr=zr+1
	data_hilf(TS)='TSP_EMS_0'+STRTRIM(zr,2)+ $
				  STRTRIM(model_time(t,0),2)+'.data'
endfor
for TS=9,TS_pa-1 do begin
	zr=zr+1
	data_hilf(TS)='TSP_EMS_'+STRTRIM(zr,2)+ $
				  STRTRIM(model_time(t,0),2)+'.data'
endfor
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;Abfangen von Anfangs- und ...
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (t eq 0) then begin
data_out=STRARR(model_time(t,1))
	j=0
	for i=modelling_start(0)-1,TS_pa-1 do begin
		data_out(j)=data_hilf(i)
		j=j+1
	endfor
endif
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ...Endverlusten
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if (t eq (model_years-1)) then begin
data_out=STRARR(model_time(t,1))
	for i=0, modelling_end(0)-1 do begin
		data_out(i)=data_hilf(i)
	endfor
endif
; ---------------------------------------------------------
; Verarbeitung und Ablage der YSP Daten: WATERVAPOR
; ---------------------------------------------------------
for i=0,(N_ELEMENTS(data_out))-1 do begin
	data_in=ALL_PFD(t,3)+'\'+data_out(i)
	dat_out=MODEL_PFD(t,4)+'\'+data_out(i)
	subset_single, data_in, dat_out, UL_xy, LR_xy, ns, nl, log
	;------------------------------------------------------
	; Log Meldung
	;------------------------------------------------------
	PRINTF,1, log
ENDFOR
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Jahresschleife
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
endfor
; ---------------------------------------------------------
; Schlieen der LOG Datei
; ---------------------------------------------------------
PRINTF,1, 'End of Logging  ', SYSTIME()
CLOSE,1
; ---------------------------------------------------------
; ffnen der LOG Datei
; ---------------------------------------------------------
command='notepad '+LOG_PFD
spawn, command
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ENDE der Routine RBM_PREP_GUI.pro
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
END